#include <iostream>
#include <fstream>
#include <tuple>
#include <vector>

#include "TGraphErrors.h"

using namespace std;

void alr()
{
   ifstream infileL("xsection.az.pole-1.dat");
   ifstream infileR("xsection.az.pole+1.dat");

   using XsecData = tuple<double,  // sqrt(s)
                          double,  // pole
                          double,  // q_lo
                          double,  // q_hi
                          double,  // sigma
                          double>; // delta sigma

   vector<XsecData> sgLdata;
   vector<XsecData> sgRdata;

   double rs, pe, qlo, qhi, sg, dsg;
   while (infileL >> rs >> pe >> qlo >> qhi >> sg >> dsg) {
      sgLdata.push_back(XsecData{rs, pe, qlo, qhi, sg, dsg});
      //cerr << qlo << " " << qhi << " " << sg << endl;
   }
   while (infileR >> rs >> pe >> qlo >> qhi >> sg >> dsg) {
      sgRdata.push_back(XsecData{rs, pe, qlo, qhi, sg, dsg});
      //cerr << qlo << " " << qhi << " " << sg << endl;
   }

   sort(sgLdata.begin(), sgLdata.end(), [](auto a, auto b) {
      auto [rsa, pea, qloa, qhia, sga, dsga] = a;
      auto [rsb, peb, qlob, qhib, sgb, dsgb] = b;
      return qloa < qlob;
   });
   sort(sgRdata.begin(), sgRdata.end(), [](auto a, auto b) {
      auto [rsa, pea, qloa, qhia, sga, dsga] = a;
      auto [rsb, peb, qlob, qhib, sgb, dsgb] = b;
      return qloa < qlob;
   });

   double xdata[100], dxdata[100], ydata[100], dydata[100];
   for (int i=0; i<sgLdata.size(); i++) {
      auto [rsL, peL, qloL, qhiL, sgL, dsgL] = sgLdata[i];
      auto [rsR, peR, qloR, qhiR, sgR, dsgR] = sgRdata[i];

      auto a  = (sgL - sgR)/(sgL + sgR);
      auto da = 0.5*(1.-a)*(1.+a)*sqrt(pow(dsgL/sgL,2)+pow(dsgR/sgR,2));
      cout << qloL << " " << qhiL << " " << a << "+/-" << da << endl;
      xdata [i] = (qhiL-qloL)/2.;
      dxdata[i] = 0;
      ydata [i] = a;
      dydata[i] = da;
   }
   auto gp = new TGraphErrors(sgLdata.size(),xdata,ydata,dxdata,dydata);
   gp->Draw("apc");

   auto t3  = -0.5;
   auto qe  = -1.0;
   auto s2w = 0.230;

   auto gl = t3 - qe*s2w;
   auto gr =    - qe*s2w;

   auto gl2 = gl*gl;
   auto gr2 = gr*gr;

   auto a0 = (gl2-gr2)/(gl2+gr2);
   cout << endl;
   cout << "alr0=" << a0 << endl;

   auto arp = new TArrow(xdata[0],a0,xdata[sgLdata.size()],a0,0.);
   arp->SetLineStyle(2);
   arp->Draw();
}
