
//
// Collection of optional macros to be used in GUIMainMacro
//
// This macro file is loaded by the function, LoadLibraries(), defined in 
// the file, $JSFROOT/macro/LoadLibraries.C, during the construction of 
// JSFSteer object.  
// The default file name of this macro file is defined by the JSFEnv varibale, 
// JSFJ4.S4MacroFileName.  The default value is $SATELLITES/macro/S4Macro.C
//
//$Id: S4Macro_RealClustering.C,v 1.3 2006/05/24 09:10:50 miyamoto Exp $

// ----------------------------------------------------------------------
void LoadS4Libraries(JSFEnv *env=0)
{
//  This function is called during the construction of JSFSteer object,
//  through the function, LoadLibraries(), defined in $JSFROOT/macro/LoadLibraries.C

  LoadLibraryWithMessage("libS4Utils.so");
  S4LibraryLoader("mctruths");
  LoadLibraryWithMessage("libJSFJ4.so");


// JSFGUI.S4Sequence: mctruths:lcio:makers:analysis
  TString inputs(env->GetValue("JSFGUI.S4Sequence",""));
  TString delim(":");
  TObjArray *tokens=inputs.Tokenize(delim);
  if( tokens->GetEntries() < 1 ) return;
  TIter next(tokens);
  TObjString *objstr=0;
  while( (objstr=(TObjString*)next()) ) {
    TString setname=objstr->GetString();
    if( setname != TString("mctruths") ) S4LibraryLoader(setname);
  }


}

// ----------------------------------------------------------------------
void ConstructS4Modules()
{
// This function is called by the function, DefineOptionalModules(), 
// which is defined in $JSFROOT/macro/OptFunctions.C. 

// JSFGUI.S4Sequence: mctruths:lcio:makers:analysis
  TString inputs(gJSF->Env()->GetValue("JSFGUI.S4Sequence",""));
  TString delim(":");
  TObjArray *tokens=inputs.Tokenize(delim);
  if( tokens->GetEntries() < 1 ) return;
  TIter next(tokens);
  TObjString *objstr=0;
  while( (objstr=(TObjString*)next()) ) {
    TString setname=objstr->GetString();
    S4ModuleDefine(setname);
  }

}

// ----------------------------------------------------------------------
void S4Initialize()
{
 // Called during the job initialization to perform the initialization 
 // task for SATELLITES.  Called from the function, Initialize(),
 // defined in $JSFROOT/macro/GUIMainMacro.C, if 
 

//----------------------------------
// set parameters
//----------------------------------
 
   S4CALHitBuf::SetHitMakingScheme(
   gJSF->Env()->GetValue("JSFJ4.CALHitMakingScheme",1)); 
	// idealistic (smeared 1 hit/1 exact hit)

   S4TPCHit::SetSpatialResolution(TString(
   gJSF->Env()->GetValue("S4TPC.SpatialResolution","experimental")));  
       // "flat" or "experimental" 
   //   S4TPCHit::SetT0Offset(0.0); //  T0 Offset

   //   S4CALCluster::SetCalibFactorForEM(37.0142);  // for mar06_2m, tower data
   //   S4CALCluster::SetCalibFactorForHD(35.2893);

   S4CALCluster::SetCalibFactorForEM(
	  gJSF->Env()->GetValue("S4CALCluster.CalibrFactorForEM",24.3367));
   S4CALCluster::SetCalibFactorForHD(
	  gJSF->Env()->GetValue("S4CALCluster.CalibrFactorForHD",33.5625));

 
//----------------------------------
// set subtrackers to hybtrackmaker
//----------------------------------
 
  S4VTXTrackMaker *vtxtrackmaker = (S4VTXTrackMaker *)gJSF->FindModule("S4VTXTrackMaker");
  S4ITTrackMaker  *ittrackmaker  = (S4ITTrackMaker *)gJSF->FindModule("S4ITTrackMaker");
  S4TPCTrackMaker *tpctrackmaker = (S4TPCTrackMaker *)gJSF->FindModule("S4TPCTrackMaker");
  S4HYBTrackMaker *htrackmaker   = (S4HYBTrackMaker *)gJSF->FindModule("S4HYBTrackMaker");

  TKalDetCradle    *cradle     = new TKalDetCradle();
  U4KalDetector    *det        = new U4KalDetector();
  U4TPCKalDetector *tpcdet     = new U4TPCKalDetector("FillLayer");
  U4ITKalDetector  *itdet      = new U4ITKalDetector("FillLayer");
  U4VTXKalDetector *vtxdet     = new U4VTXKalDetector("FillLayer");

  tpctrackmaker->SetDetector(*tpcdet);
  ittrackmaker ->SetDetector(*itdet);
  vtxtrackmaker->SetDetector(*vtxdet);
  htrackmaker->SetDetector(*det);

  cradle->Install(*det);
  cradle->Install(*tpcdet);
  cradle->Install(*itdet);
  cradle->Install(*vtxdet);
  cradle->Close();

  htrackmaker->SetMainTrackMakerPtr(tpctrackmaker);
  htrackmaker->AddSubTrackMakerPtr(ittrackmaker);
  htrackmaker->AddSubTrackMakerPtr(vtxtrackmaker);

}

// ----------------------------------------------------------------------
void S4LibraryLoader(TString setname)
{
  cerr  << "Loading libraries for set " << setname << endl;
  if( setname == TString("lcio") ) {
    LoadLibraryWithMessage("libJSF2LCIO.so");
    LoadLibraryWithMessage("libJ42LCIO.so");
  }
  else if ( setname == TString("mctruths") ) { 
    LoadLibraryWithMessage("libS4MCTruth.so");
    LoadLibraryWithMessage("libS4GENMCTruth.so");
    LoadLibraryWithMessage("libS4VTXMCTruth.so");
    LoadLibraryWithMessage("libS4ITMCTruth.so");
    LoadLibraryWithMessage("libS4TPCMCTruth.so");
    LoadLibraryWithMessage("libS4CDCMCTruth.so");
    LoadLibraryWithMessage("libS4CALMCTruth.so");
    LoadLibraryWithMessage("libS4MUDMCTruth.so");
    LoadLibraryWithMessage("libS4FCALMCTruth.so");
  }
  else if ( setname == TString("makers") ) {
    LoadLibraryWithMessage("libS4Kalman.so");
    LoadLibraryWithMessage("libS4Geom.so");
    LoadLibraryWithMessage("libS4KalTrack.so");
    LoadLibraryWithMessage("libU4VHitMaker.so");
    LoadLibraryWithMessage("libU4KalDetector.so");
    LoadLibraryWithMessage("libU4VTXKalDetector.so");
    LoadLibraryWithMessage("libU4ITKalDetector.so");
    LoadLibraryWithMessage("libU4TPCKalDetector.so");

    LoadLibraryWithMessage("libU4TPCHitMaker.so");
    LoadLibraryWithMessage("libS4TPCHitMaker.so");
    LoadLibraryWithMessage("libU4ITHitMaker.so");
    LoadLibraryWithMessage("libS4ITHitMaker.so");
    LoadLibraryWithMessage("libU4VTXHitMaker.so");
    LoadLibraryWithMessage("libS4VTXHitMaker.so");

    LoadLibraryWithMessage("libU4VTrackMaker.so");
    LoadLibraryWithMessage("libS4Kern.so");
    LoadLibraryWithMessage("libU4TPCTrackMaker.so");
    LoadLibraryWithMessage("libS4TPCTrackMaker.so");
    LoadLibraryWithMessage("libU4ITTrackMaker.so");
    LoadLibraryWithMessage("libS4ITTrackMaker.so");
    LoadLibraryWithMessage("libU4VTXTrackMaker.so");
    LoadLibraryWithMessage("libS4VTXTrackMaker.so");
    LoadLibraryWithMessage("libU4HYBTrackMaker.so");
    LoadLibraryWithMessage("libS4HYBTrackMaker.so");

    LoadLibraryWithMessage("libS4CALHitMaker.so");
    LoadLibraryWithMessage("libS4CALSmallClusterMaker.so");
    LoadLibraryWithMessage("libS4CALClusterMaker.so");
    LoadLibraryWithMessage("libS4CALRealClusterMaker.so");

    LoadLibraryWithMessage("libAnlib.so");
    LoadLibraryWithMessage("libS4PFOMaker.so");
    LoadLibraryWithMessage("libS4JetMaker.so");
  }
  else if ( setname == TString("analysis") ) {
    LoadLibraryWithMessage("libS4TPCAnalysis.so");
    LoadLibraryWithMessage("libS4ITAnalysis.so");
    LoadLibraryWithMessage("libS4VTXAnalysis.so");
    LoadLibraryWithMessage("libS4CALAnalysis.so");
    LoadLibraryWithMessage("libS4PFOAnalysis.so");
    LoadLibraryWithMessage("libS4JetAnalysis.so");
  }
  else {
    cerr << "Undefined library set name " << setname << " was used in S4LibraryLoader() " << endl;
    gSystem->Exit(-1);
  }

}

// ----------------------------------------------------------------------
void S4ModuleDefine(TString setname)
{
  cerr << " Define modules for set " << setname << endl;

  if( setname == TString("lcio") ) {
    J42LCIO *j42lcio=new J42LCIO();
    j42lcio -> SetMakeBranch(kFALSE);
  }
  else if ( setname == TString("mctruths") ) {
    JSFJ4BufManager *bufman  = new JSFJ4BufManager();  
    S4GENMCTruth  *genmctruth   = new S4GENMCTruth();  
    S4VTXMCTruth  *vtxmctruth   = new S4VTXMCTruth();  
    S4ITMCTruth   *itmctruth    = new S4ITMCTruth();   
    S4TPCMCTruth  *tpcmctruth   = new S4TPCMCTruth();  
    S4CALMCTruth  *calmctruth   = new S4CALMCTruth();  

//  S4MUDMCTruth  *mudmctruth   = new S4MUDMCTruth();  

    bufman     -> SetMakeBranch(kFALSE);
    genmctruth -> SetMakeBranch(kFALSE);
    vtxmctruth -> SetMakeBranch(kFALSE);
    itmctruth  -> SetMakeBranch(kFALSE);
    tpcmctruth -> SetMakeBranch(kFALSE);
    calmctruth -> SetMakeBranch(kFALSE);
    
//    mudmctruth -> SetMakeBranch(kFALSE);

  }
  else if ( setname == TString("makers") ) {
//----------------------------------
// Define KalDetector
//----------------------------------
     TKalDetCradle    *cradle     = new TKalDetCradle();
     U4KalDetector    *det        = new U4KalDetector();
     U4TPCKalDetector *tpcdet     = new U4TPCKalDetector("FillLayer");
     U4ITKalDetector  *itdet      = new U4ITKalDetector("FillLayer");
     U4VTXKalDetector *vtxdet     = new U4VTXKalDetector("FillLayer");
     cradle->Install(*det);
     cradle->Install(*vtxdet);
     cradle->Install(*itdet);
     cradle->Install(*tpcdet);

     S4TPCHitMaker     *tpchitmaker   = new S4TPCHitMaker();   
     S4TPCTrackMaker   *tpctrackmaker = new S4TPCTrackMaker(tpcdet);
     S4ITHitMaker      *ithitmaker    = new S4ITHitMaker();
     S4ITTrackMaker    *ittrackmaker  = new S4ITTrackMaker(itdet);
     S4VTXHitMaker     *vtxhitmaker   = new S4VTXHitMaker();
     S4VTXTrackMaker   *vtxtrackmaker = new S4VTXTrackMaker(vtxdet);
     S4HYBTrackMaker   *htrackmaker   = new S4HYBTrackMaker(det);
                                                                                
     S4CALHitMaker     *calhitmaker   = new S4CALHitMaker();
     S4CALSmallClusterMaker *calsmallclustmaker = new S4CALSmallClusterMaker();
     S4CALRealClusterMaker  *calrealclustmaker  = new S4CALRealClusterMaker();
     //S4CALClusterMaker *calclustmaker = new S4CALClusterMaker();
     //S4PFOMaker        *pfomaker      = new S4PFOMaker();
     //S4JetMaker        *jetmaker      = new S4JetMaker();
                                                                                
     tpchitmaker   -> SetMakeBranch(kFALSE);
     tpctrackmaker -> SetMakeBranch(kFALSE);
     ithitmaker    -> SetMakeBranch(kFALSE);
     ittrackmaker  -> SetMakeBranch(kFALSE);
     vtxhitmaker    -> SetMakeBranch(kFALSE);
     vtxtrackmaker  -> SetMakeBranch(kFALSE);
     htrackmaker   -> SetMakeBranch(kFALSE);
                                                                                
     calhitmaker   -> SetMakeBranch(kFALSE);
     calsmallclustmaker -> SetMakeBranch(kFALSE);
     calrealclustmaker -> SetMakeBranch(kFALSE);
     //calclustmaker -> SetMakeBranch(kFALSE);
     //pfomaker      -> SetMakeBranch(kFALSE);
     //jetmaker      -> SetMakeBranch(kFALSE);
                                                                                
   }
   else if ( setname == TString("analysis") ) {
     S4TPCAnalysis     *tpcanalysis   = new S4TPCAnalysis();
     S4ITAnalysis      *itanalysis    = new S4ITAnalysis();
     S4VTXAnalysis     *vtxanalysis   = new S4VTXAnalysis();
     //S4CALAnalysis     *calanalysis   = new S4CALAnalysis();
     //S4PFOAnalysis     *pfoanalysis   = new S4PFOAnalysis();
     //S4JetAnalysis     *jetanalysis   = new S4JetAnalysis();

     tpcanalysis   -> SetMakeBranch(kFALSE);
     itanalysis    -> SetMakeBranch(kFALSE);
     vtxanalysis   -> SetMakeBranch(kFALSE);
     //calanalysis   -> SetMakeBranch(kFALSE);
     //pfoanalysis   -> SetMakeBranch(kFALSE);
     //jetanalysis   -> SetMakeBranch(kFALSE);

   } 
}
