//
// cdcanl.C
//
// 

#include <iomanip>

void cdcanl()
{
//----------------------------------
// Reset ROOT
//----------------------------------

   gROOT->Reset();

   gErrorAbortLevel = kWarning + 1; 

//----------------------------------
// Create JSF runtime environment
//----------------------------------

   JSFSteer *jsfp = new JSFSteer();

//----------------------------------
// Load user defined classes
//----------------------------------

   gSystem->Load("libS4Kalman.so");  
   gSystem->Load("libS4Geom.so");  
   gSystem->Load("libS4KalTrack.so");  
   gSystem->Load("libS4GENMCTruth.so");
   gSystem->Load("libU4CDCRawData.so");
   gSystem->Load("libU4CDCClusterMaker.so");
   gSystem->Load("libU4VHitMaker.so");
   gSystem->Load("libU4KalDetector.so");  
   gSystem->Load("libU4CDCKalDetector.so");  
   gSystem->Load("libU4CDCHitMaker.so");
   gSystem->Load("libS4CDCMCTruth.so");
   gSystem->Load("libU4VTrackMaker.so");  
   gSystem->Load("libS4Kern.so");
   gSystem->Load("libS4CDCHitMaker.so");
   gSystem->Load("libU4CDCTrackMaker.so");  
   gSystem->Load("libS4CDCTrackMaker.so");  
   gSystem->Load("libU4HYBTrackMaker.so");
   gSystem->Load("libS4HYBTrackMaker.so");
   gSystem->Load("libS4CDCAnalysis.so");
  
//----------------------------------
// Set output file
//----------------------------------

   TFile *foutp = new TFile("jsf.cdcanl.root","RECREATE"); //  Output file.
   TFile &fout  = *foutp;

   jsfp->SetOutput(fout);

//----------------------------------
// Set input file
//----------------------------------

   TFile file("jsfj4.cdc.root");
   file.ls();
   jsfp->SetInput(file);

   Int_t nevent = jsfp->Env()->GetValue("JSFSteer.Nevent",999999);  
   Int_t minevt = 1;
   Int_t maxevt = minevt + nevent;

//----------------------------------
// Define KalDetector
//----------------------------------
#if 1
   // Axial Only Mode (should set before creating U4CDCKalDetector)
   //===================================================================
                 U4CDCKalDetector::SetAxialOnly();
   //===================================================================
#endif
   U4CDCKalDetector *cdcdet      = new U4CDCKalDetector();

//----------------------------------
// Insert modules
//----------------------------------
 
   S4GENMCTruth    *mctruthgen  = new S4GENMCTruth();
   S4CDCHitMaker   *hitmaker    = new S4CDCHitMaker();
   S4CDCTrackMaker *trackmaker  = new S4CDCTrackMaker(cdcdet);
   S4HYBTrackMaker *htrackmaker = new S4HYBTrackMaker(cdcdet);
   S4CDCAnalysis   *analysis    = new S4CDCAnalysis();

//----------------------------------
// Set parameters
//----------------------------------

   // Remove Dalta-Ray hits ....    
#if 0
#if 0
      mctruth->RemoveDeltaRayHits(kFALSE);
      mctruth->JoinSeparatedHits(kFALSE);
#else
      mctruth->RemoveDeltaRayHits(kTRUE);
      mctruth->JoinSeparatedHits(kTRUE);
#endif
#endif

   // Saturation effect ....    
      analysis->SaturationOff();
 
   // Two-track Separation ....
      Int_t twotrack = jsfp->Env()->GetValue("JSFSteer.TwoTrackSeparation",2.0);  
      Double_t twotracksep = twotrack/10.; // [cm]
      S4CDCHitWire::SetTwoHitSeparation(twotracksep);
      cerr << " twotracksep = " << twotrack << "[mm]" << endl;  
#if 0
      S4CDCHitWire::SetTwoHitSeparationZ(0.3); // [cm]
#endif
#if 0
      S4CDCHit::SetSpatialResolution("experimental"); // default is "flat".
#endif
 
   // Set IP constraint ....
#if 0
      S4CDCTrack::SetIPConstraint(kTRUE);
#endif

   // Artificial bunch tagging
      Double_t t0 = 14.;
      S4CDCTrack::SetT0pedestal(t0); // [nsec]
     
   // Simulation Mode (CDC or TPC) ....
      S4CDCHitWire::TPCMode(kFALSE);

//----------------------------------
// Job initialization
//----------------------------------
 
   jsfp->Initialize();

//----------------------------------
// set subtrackers to hybtrackmaker 
//----------------------------------

  htrackmaker->SetMainTrackMakerPtr(trackmaker);
  //htrackmaker->AddSubTrackMakerPtr(ittrackmaker);
  //htrackmaker->AddSubTrackMakerPtr(vtxtrackmaker);

//----------------------------------
// Start event loop
//----------------------------------

   for(Int_t ev = minevt; ev < maxevt; ev++) {
      if (!jsfp->GetEvent(ev)) break;
      cerr << " ---- Event " << jsfp->GetEventNumber() << " ----" << endl;
      if (!jsfp->Process(ev)) {
         cerr << "jsfp->Process is faild." << endl;
         jsfp->FillTree();
         jsfp->Clear();
         break;
      }
      jsfp->FillTree();
      jsfp->Clear();
   }
   jsfp->Terminate();
   fout.Write();
}
