//*************************************************************************
//* --------------------
//* class S4HISTOutput
//* --------------------
//* (Description)
//*     Class to output break points.
//* (Update Record)
//*     2006/01/29  S.Yamamoto     Original version.
//*************************************************************************

#include "J4BreakPoint.hh"
#include "S4BreakPoint.h"
#include "S4HISTOutput.h"
#include "TObjArray.h"
#include <map>
#include <iostream>

//=====================================================================
//* Constructor -------------------------------------------------------
S4HISTOutput::S4HISTOutput(TObjArray *bufp) : fOutBufPtr(bufp)
{
  // Set itself as a J4Output object to be used to output
  // break point information in J4BreakPoint::Output().
  // J4BreakPoint::Output() is called at the end of each event
  // from J4EventAction::EndOfEventAction(...).
  //  J4BreakPoint::SetOutput(this);

  J4BreakPoint::SetOutput(this);

}


//=====================================================================
//* Destructor -------------------------------------------------------
S4HISTOutput::~S4HISTOutput()
{
}

//=====================================================================
//* Output ------------------------------------------------------------
void S4HISTOutput::Output(J4VHit *)
{
  // The function overrides the base class' Output() and enables
  // J4BreakPoint::Output() to output break point information into
  // an output root file.
  typedef std::map<G4int, J4BreakPoint*> BPMap_t;
  BPMap_t           &bpmap = J4BreakPoint::GetBreakPointMap();
  BPMap_t::iterator  iter;
  for (iter  = bpmap.begin();
       iter != bpmap.end(); iter++) {
    fOutBufPtr->Add(new S4BreakPoint(iter->second));
  }
}

//=====================================================================
//* Clear -------------------------------------------------------------
void S4HISTOutput::Clear()
{
  if (fOutBufPtr) fOutBufPtr->Clear();
}

//=====================================================================
//* Delete ------------------------------------------------------------
void S4HISTOutput::Delete()
{
  if (fOutBufPtr) fOutBufPtr->Delete();
}
