//*************************************************************************
//* ----------------
//* S4BabyMCTruth Class
//* ----------------
//* (Description)
//* (Requires)
//*	TObject:
//*	TClonesArray:
//*	BabyUniversalRecordHeader:
//*	BabyRawADCDatum:
//* (Provides)
//*	S4BabyMCTruthBuf		: raw data buffer
//*	S4BabyMCTruth			: steering class
//* (Update Record)
//*	2002/08/27  R.Kuboshima
//*************************************************************************

#include "S4BabyMCTruth.h"
#include "S4BabyMCTruthBuf.h"
#include "TRandom.h"
#include <iostream>

ClassImp(S4BabyMCTruth)


//___________________________________________________________________________
// -----------------
// MCTruth Analysis
// -----------------
//
S4BabyMCTruth::S4BabyMCTruth(const char *name, const char *title)
             : S4VMCTruth(name,title),
               fRemoveDeltaRayHits(kFALSE), fJoinSeparatedHits(kTRUE)
{
    std::cerr << " constructor for S4BabyMCTruth is called..." << std::endl;
    fEventBuf = new S4BabyMCTruthBuf(this);
}

//___________________________________________________________________________
S4BabyMCTruth::~S4BabyMCTruth()
{
    std::cerr << " S4BabyMCTruth will be deleted...fEventBuf is " 
    	 << (Int_t)fEventBuf << std::endl;
    if(fEventBuf)     delete fEventBuf;
    if(S4VMCTruth::GetInputStreamPtr()) S4VMCTruth::CloseInputStream();
}

//___________________________________________________________________________
Bool_t S4BabyMCTruth::BeginRun(Int_t nrun)
{
    return S4VMCTruth::BeginRun(nrun);
}

//___________________________________________________________________________
Bool_t S4BabyMCTruth::Process(Int_t ev)
{
    S4BabyMCTruthBuf *buf = (S4BabyMCTruthBuf*)fEventBuf;
    buf->GetExactHitsPtr()->Delete();

    if ( fFile->IsWritable() ) {
    	buf->ReadFile();
    	std::cerr << " S4BabyMCTruth::Process nhits = " 
    	     << buf->GetExactHitsPtr()->GetEntries() << std::endl;
    	if (fRemoveDeltaRayHits) {
    		buf->RemoveDeltaRayHits();
    		std::cerr << " S4BabyMCTruth::Process nhits after compress = " 
    		     << buf->GetExactHitsPtr()->GetEntries() << std::endl;
    		
	}
    	if (fJoinSeparatedHits) {
    		buf->JoinSeparatedHits();
	}
    	return kTRUE;
    } else {
        std::cerr << "S4BabyMCTruth::Process ExacthitsBuffer is not updeted. " << std::endl;
    	return kTRUE;
    }
}


//___________________________________________________________________________
void S4BabyMCTruth::DebugPrint(const Char_t *)
{
}


