//$Id: S4CALTowerGeom.cxx,v 1.1 2005/08/04 02:51:46 miyamoto Exp $
//****************************************************
//*
//* ---------------------
//* S4CALTowerGeom
//* ---------------------
//*   Stores CAL geometry data
//*
//* (Update Record)
//*   2005/07/29  A.Miyamoto  Original version
//****************************************************

#include <iostream>
#include "S4CALTowerGeom.h"
#include "J4CALParameterList.hh"

// ------------------------------
S4CALTowerGeom::S4CALTowerGeom():
  fNCones(0), fRadius(0), fTheta(0), fDTheta(0), fNPhi(0)
{
}

// ------------------------------
S4CALTowerGeom::~S4CALTowerGeom()
{
  if( fRadius ) delete fRadius;
  if( fTheta )  delete fTheta;
  if( fDTheta ) delete fDTheta;
  if( fNPhi )   delete fNPhi;

}

// ------------------------------
void S4CALTowerGeom::LoadJ4CALParameterList()
{
  // Load Tower geometry data in J4CALParameterList 
  // to data member
  J4CALParameterList *pl=J4CALParameterList::GetInstance();

  fNCones = pl->GetNcones();
  fRadius = new Double_t[fNCones];
  fTheta  = new Double_t[fNCones];
  fDTheta = new Double_t[fNCones];
  fNPhi   = new Int_t[fNCones];
  for(Int_t i=0; i< GetNCones() ; i++ ) {
    fRadius[i] = pl->GetTowerParam(i)->GetR()/10.0;
    fTheta[i]  = pl->GetTowerParam(i)->GetTheta();
    fDTheta[i] = pl->GetTowerParam(i)->GetDlambda();
    fNPhi[i]   = pl->GetTowerParam(i)->GetNphi();
  }

}

// ------------------------------
void S4CALTowerGeom::Print()
{
  std::cout << "CAL Tower Geometry : Number of cones =" << fNCones << std::endl;
  std::cout << " i  Radius  Theta  DTheta  NPhi " << std::endl;

  for(Int_t i=0;i<fNCones;i++) {
    std::cout << i << " " ;
    std::cout << GetRadius(i) << " " ;
    std::cout << GetTheta(i) << " " ;
    std::cout << GetDTheta(i) << " " ;
    std::cout << GetNPhi(i) ;
    std::cout << std::endl;
  }

}

