//*************************************************************************
//* ----------------
//* S4CDCMCTruth Class
//* ----------------
//* (Description)
//* (Requires)
//*	TObject:
//*	TClonesArray:
//*	CDCUniversalRecordHeader:
//*	CDCRawADCDatum:
//* (Provides)
//*	S4CDCMCTruthBuf		: raw data buffer
//*	S4CDCMCTruth			: steering class
//* (Update Record)
//*	1999/06/02  K.Fujii	Original version.
//*************************************************************************

#include "S4CDCMCTruth.h"
#include "S4CDCMCTruthBuf.h"
#include <iostream>

ClassImp(S4CDCMCTruth)


//___________________________________________________________________________
// -----------------
// MCTruth Analysis
// -----------------
//
S4CDCMCTruth::S4CDCMCTruth( const char *name, const char *title)
             : S4VMCTruth(name,title),
               fRemoveDeltaRayHits(kFALSE), fJoinSeparatedHits(kTRUE)
{
    std::cerr << " constructor for S4CDCMCTruth is called..." << std::endl;
    fEventBuf = new S4CDCMCTruthBuf(this);
}

//___________________________________________________________________________
S4CDCMCTruth::~S4CDCMCTruth()
{
    std::cerr << " S4CDCMCTruth will be deleted...fEventBuf is " 
    	 << (void *)fEventBuf << std::endl;
    if (fEventBuf)     delete fEventBuf;
    if (S4VMCTruth::GetInputStreamPtr()) S4VMCTruth::CloseInputStream();
}

//___________________________________________________________________________
Bool_t S4CDCMCTruth::BeginRun(Int_t nrun)
{
    return S4VMCTruth::BeginRun(nrun);
}

//___________________________________________________________________________
Bool_t S4CDCMCTruth::Process(Int_t ev)
{
    S4VMCTruth::Process(ev);

    S4CDCMCTruthBuf *buf = (S4CDCMCTruthBuf*)fEventBuf;

    if ( fFile->IsWritable() ) {
       if (buf->GetInputBufPtr()) {
          TObjArray *ehitbufp = buf->GetExactHitsPtr();
          ehitbufp->Clear();
          TIter next(buf->GetInputBufPtr());
          TObject *objp = 0;
          while ((objp = next())) ehitbufp->Add(objp);
       } else {
          buf->GetExactHitsPtr()->Delete();
          buf->GetTmpExactHitsPtr()->Delete();
          buf->ReadFile();
       }
#ifdef __DEBUG__
       std::cerr << " S4CDCMCTruth::Process nhits = " 
                 << buf->GetExactHitsPtr()->GetEntries() << std::endl;
#endif
       if (fRemoveDeltaRayHits) {
          buf->RemoveDeltaRayHits();
          std::cerr << " S4CDCMCTruth::Process nhits after compress = " 
                    << buf->GetExactHitsPtr()->GetEntries() << std::endl;
       }
       if (fJoinSeparatedHits) {
          buf->JoinSeparatedHits();
       }
       return kTRUE;
    } else {
        std::cerr << "S4CDCMCTruth::Process ExacthitsBuffer is not updeted. " << std::endl;
    	return kTRUE;
    }
}


//___________________________________________________________________________
void S4CDCMCTruth::DebugPrint(const Char_t *)
{
}
