//*************************************************************************
//* ----------------
//* S4ITMCTruth Class
//* ----------------
//* (Description)
//* (Requires)
//*	TObject:
//*	TClonesArray:
//*	ITUniversalRecordHeader:
//*	ITRawADCDatum:
//* (Provides)
//*	S4ITMCTruthBuf		: raw data buffer
//*	S4ITMCTruth		: steering class
//* (Update Record)
//*	2004/06/02  A.Yamaguchi	Original version.
//*************************************************************************

#include <iostream>
#include "S4ITMCTruth.h"
#include "S4ITExactHit.h"
#include "S4ITMCTruthBuf.h"
using namespace std;

//#define __DEBUG__

ClassImp(S4ITMCTruth)

//=====================================================================
//* constructor -------------------------------------------------------

S4ITMCTruth::S4ITMCTruth(const Char_t *fname, 
                         const Char_t *name,
                         const Char_t *title)
           : S4VMCTruth(name, title),
             fInputFileName((Char_t *)fname)
{
#ifdef __DEBUG__
   cerr << " constructor for S4ITMCTruth is called..." << endl;
#endif
   fEventBuf = new S4ITMCTruthBuf(this);
}

//=====================================================================
//* destructor --------------------------------------------------------

S4ITMCTruth::~S4ITMCTruth()
{
#ifdef __DEBUG__
   cerr << " S4ITMCTruth will be deleted...fEventBuf is " 
        << (Int_t)fEventBuf << endl;
#endif
   if(fEventBuf) delete fEventBuf;
   if(!fInputStream) fInputStream->close();
}

//=====================================================================
//* BeginRun  ---------------------------------------------------------
                                                                                
Bool_t S4ITMCTruth::BeginRun(Int_t nrun)
{
#ifdef __DEBUG__
   cerr << "S4ITMCTruth::BeginRun() is called..." << endl;
#endif
    return S4VMCTruth::BeginRun(nrun);
}

//=====================================================================
//* Process   --------------------------------------------------------

Bool_t S4ITMCTruth::Process(Int_t ev)
{
   S4VMCTruth::Process(ev);
#ifdef __DEBUG__
   cerr << "S4IT`MCTruth::Process() is called..." << endl;  
#endif
   S4ITMCTruthBuf *bufp = dynamic_cast<S4ITMCTruthBuf *>(fEventBuf);

   if ( fFile->IsWritable() ) {
      if (bufp->GetInputBufPtr()) {
         bufp->GetExactHitsPtr()->Clear();
         TObjArray *ehitbufp   = bufp->GetExactHitsPtr(); 
         TIter next(bufp->GetInputBufPtr());
         TObject *objp = 0;
         while ((objp = next())) ehitbufp->Add(objp);
#ifdef __DEBUG__
         cerr << "ITHitsPtr = " << (void *)bufp->GetInputBufPtr() << endl;
         cerr << "Nhits     = " << bufp->GetInputBufPtr()->GetEntries() << endl;
#endif
      } else {
         bufp->GetExactHitsPtr()->Delete();
         bufp->ReadFile();
      }
#ifdef __DEBUG__
      cerr << "S4ITMCTruth::Process nhits =" 
           << bufp->GetExactHitsPtr()->GetEntries() << endl;
#endif
      bufp->MergeHits();
#ifdef __DEBUG__
      DebugPrint();
#endif
      return kTRUE;
   }
   else {
      cerr << "S4ITMCTruth::Process ExacthitsBuffer is not updated. " << endl;
      return kTRUE;
   }
}


//=====================================================================
//* DebugPrint -------------------------------------------------------

void S4ITMCTruth::DebugPrint(const Char_t *)
{
   S4ITMCTruthBuf *bufp = (S4ITMCTruthBuf *)fEventBuf;
   TIter next(bufp->GetExactHitsPtr()); 
   S4ITExactHit *ehitp;
   while ((ehitp = static_cast<S4ITExactHit *>(next()))) {
      ehitp->DebugPrint();
   }
}
