#ifndef __S4CALHITMAKER__
#define __S4CALHITMAKER__
//$Id: S4CALHitMaker.h,v 1.7 2005/10/27 03:08:23 fujikawa Exp $
//*****************************************************************************
//* ==============
//* S4CALHitMaker
//* ==============
//*   METIS module to make (smeared) CAL hits.
//*   Mother class: S4VHit
//*
//* (Update Record)
//*   2004.07.31  A.Sanchez  Primitive version.
//*****************************************************************************
#include "JSFModule.h"
#include "TObjArray.h"
#include "S4CALHit.h"

class S4CALHitMaker;



//****************** S4CALHitBuf class ****************************************
class S4CALHitBuf : public JSFEventBuf {

public:
  S4CALHitBuf(S4CALHitMaker* hitmaker = 0,
	      const Char_t * name     = "S4CALHitBuf",
	      const Char_t * title    = "S4CALHitMaker Buf",
                    Int_t    bufsize  = 10000);
  virtual ~S4CALHitBuf();

  Int_t      GetNhits() { return fOutputBufPtr->GetEntries(); }
  TObjArray* GetHits() { return fOutputBufPtr; }
  TObjArray *GetOutputBufPtr() { return fOutputBufPtr; }
  S4CALHit*  GetHit(Int_t i=0) { return dynamic_cast<S4CALHit*>(fOutputBufPtr->At(i)); }

  static Int_t GetHitMakingScheme()        { return gfScheme; }
  static void  SetHitMakingScheme(Int_t s) { gfScheme = s; }

  void   SetInputDataBufPtr(JSFEventBuf *buf) { fInputDataBufPtr = buf; }

  void   AddHit(S4CALHit *hit) { fOutputBufPtr->Add(hit); }
  Bool_t MakeHits(TObjArray *hits);
  void   MergeHits(TObjArray *hits);
  void   SortHits(Int_t option=1);
  void   SetThreshold(Double_t thr) { fThreshold = thr; }

private:
  void SmearHits();
  void ApplyThreshold();

private:
  Double_t        fThreshold;        // threshold
  TObjArray      *fOutputBufPtr;     // output buffer ptr
  JSFEventBuf    *fInputDataBufPtr;  // Input data buffer
  Int_t           fBufSize;          // output buffer size
  static Int_t    gfScheme;          // (0,1) = (realistic, idealistic)
  static Double_t fgDT;              // resolution of TOF (nsec)

  ClassDef(S4CALHitBuf, 1) // CAL (smeared) hit buffer
};

//****************** S4CALHitMaker class ************************************

class S4CALHitMaker : public JSFModule {

public:

  S4CALHitMaker(const Char_t *name="S4CALHitMaker",
		const Char_t *title="METIS CAL Hit Maker",
		const Char_t *opt="");
  virtual ~S4CALHitMaker();

  Bool_t Initialize();
  Bool_t Process(Int_t event);

  ClassDef(S4CALHitMaker, 1) // METIS CAL Hit Maker
};


#endif
