#ifndef __S4CALCHDCLUSTERMAKER__
#define __S4CALCHDCLUSTERMAKER__

//*************************************************************************   
//* ---------------------                      
//* S4CALCHDClusterMaker Class
//* ---------------------   
//* (Description) 
//*   make charged hadron clusters
//* (Requires)
//*
//* (Provides)  
//*
//* (Update Record)
//*
//*
//*************************************************************************

#include "S4CALRealCluster.h"
#include "S4CALSmallCluster.h"
#include "S4HYBTrack.h"
#include "TObject.h"

#include <utility>
#include <map>
#include <vector>

class S4CALCHDClusterMaker {

 public:

  S4CALCHDClusterMaker();
  virtual ~S4CALCHDClusterMaker();


 public:

 //---------------------------------------------------------
 // make charged hadron clusters
 //---------------------------------------------------------
  std::map<Int_t,S4CALHit*> MakeCHDClusters(std::map<Int_t,S4CALRealCluster*> &clmap,
					    std::map<Int_t,S4CALSmallCluster*> &sclmap,
					    std::map<Int_t,S4HYBTrack*> &trkmap);

  std::vector<Int_t> DefiniteCHDIs(std::map<Int_t,S4CALSmallCluster*> &sclmap,
				   std::map<Int_t,S4HYBTrack*> &trkmap);

 //=================================================================
 //* Cut values
 //-----------------------------------------------------------------
 
 public:

 virtual Double_t GetCALResolution()     { return fCALResolution;     }
 virtual Double_t GetMaxDeviation()      { return fMaxDeviation;      }
 virtual Double_t GetEMTrackDisCutRmin() { return fEMTrackDisCutRmin; }
 virtual Double_t GetEMTrackDisCutRmax() { return fEMTrackDisCutRmax; }
 virtual Double_t GetHDTrackDisCutRmin() { return fHDTrackDisCutRmin; }
 virtual Double_t GetHDTrackDisCutRmax() { return fHDTrackDisCutRmax; }
 virtual Double_t GetEMTrackDisCutStep() { return fEMTrackDisCutStep; }


 public:

 virtual void SetCALResolution(Double_t res)      { fCALResolution     = res;}
 virtual void SetMaxDeviation(Double_t nsig)      { fMaxDeviation      = nsig;}
 virtual void SetEMTrackDisCutRmin(Double_t rmin) { fEMTrackDisCutRmin = rmin;}
 virtual void SetEMTrackDisCutRmax(Double_t rmax) { fEMTrackDisCutRmax = rmax;}
 virtual void SetHDTrackDisCutRmin(Double_t rmin) { fHDTrackDisCutRmin = rmin;}
 virtual void SetHDTrackDisCutRmax(Double_t rmax) { fHDTrackDisCutRmax = rmax;}
 virtual void SetEMTrackDisCutStep(Double_t step) { fEMTrackDisCutStep = step;}


 private:

 Double_t fCALResolution;     // calorimeter energy resolution (sigma/sqrt(E)%)
 Double_t fMaxDeviation;      // maximum cluster energy defference from track
 Double_t fEMTrackDisCutRmin; // starting value for track ditance cut (cm)
 Double_t fEMTrackDisCutRmax; // upper limit for track distance cut (cm)
 Double_t fHDTrackDisCutRmin; // starting value for track distance cut (cm)
 Double_t fHDTrackDisCutRmax; // upper limit for track distance cut (cm)
 Double_t fEMTrackDisCutStep; // step size for track tube matcing (cm)


  ClassDef(S4CALCHDClusterMaker, 1)
};
    
#endif
