#ifndef __S4CALEMCLUSTERMAKER__
#define __S4CALEMCLUSTERMAKER__

//*************************************************************************   
//* ---------------------                      
//* S4CALEMClusterMaker Class
//* ---------------------   
//* (Description) 
//*     represent calorimeter cluster
//* (Requires)
//*
//* (Provides)  
//*
//* (Update Record)
//*
//*
//*************************************************************************

#include "S4CALRealCluster.h"
#include "S4CALSmallCluster.h"
#include "S4HYBTrack.h"
#include "TObject.h"

#include <utility>
#include <map>
#include <vector>

class S4CALEMClusterMaker {

 public:
  S4CALEMClusterMaker();
  virtual ~S4CALEMClusterMaker();
  
 public:
  //---------------------------------------------------------
  // Add gamma/electron origined small clusters into clmap.
  //---------------------------------------------------------
  void MakeEMClusters(std::map<Int_t, S4CALRealCluster*> &clmap,
		      std::map<Int_t, S4CALSmallCluster*> &sclmap,
		      std::map<Int_t,S4HYBTrack*> &trkmap);
  
  //---------------------------------------------------------
  // Add gamma origined small clusters into clmap.
  // The small clusters around mother will be collected.
  //---------------------------------------------------------
  void MakePhotonClusters(std::map<Int_t, S4CALRealCluster*> &clmap,
			  std::map<Int_t, S4CALSmallCluster*> &sclmap,
			  std::map<Int_t,S4HYBTrack*> &trkmap);
  
  //---------------------------------------------------------
  // Add gamma origined small clusters into clmap.
  // Target is low energy gamma.
  //---------------------------------------------------------
  void MakeSmallPhotonClusters(std::map<Int_t, S4CALRealCluster*> &clmap,
			       std::map<Int_t, S4CALSmallCluster*> &sclmap,
			       std::map<Int_t,S4HYBTrack*> &trkmap);
  
  //---------------------------------------------------------
  // Add electron origined small clusters into clmap.
  // This method shold be performed after photon clustering.
  //---------------------------------------------------------
  void MakeElectronClusters(std::map<Int_t, S4CALRealCluster*> &clmap,
			    std::map<Int_t, S4CALSmallCluster*> &sclmap,
			    std::map<Int_t,S4HYBTrack*> &trkmap);
  
  //=================================================================
  //* Cut values
  //-----------------------------------------------------------------
  
 public:
  
  Double_t GetIsolationRin()        { return fIsolationRin;        }
  Double_t GetIsolationRout()       { return fIsolationRout;       }
  Double_t GetIsolationTubeHalfH()  { return fIsolationTubeHalfH;  }
  Double_t GetIsolationCut()        { return fIsolationCut;        }
  
  Double_t GetChisquareCut()        { return fChisquareCut;        }
  Double_t GetFitterTubeRCut()      { return fFitterTubeRCut;      }
  Double_t GetFitterTubeHalfHCut()  { return fFitterTubeHalfHCut;  }
  
  Double_t GetTrackDistanceCut()    { return fTrackDistanceCut;    }
  
  Double_t GetEHCompareTubeR()      { return fEHCompareTubeR;      }
  Double_t GetEHCompareTubeHalfH()  { return fEHCompareTubeHalfH;  }
  Double_t GetEHCompareHERatioCut() { return fEHCompareHERatioCut; }
  
  Double_t GetTubeRCut()            { return fTubeRCut;            }
  Double_t GetTubeHalfHCut()        { return fTubeHalfHCut;        }
  Double_t GetTOFCut()              { return fTOFCut;              }
  
  Double_t GetMinLayerCut()         { return fMinLayerCut;         }
  
 public:
  
  void SetIsolationRin(Double_t rin)          { fIsolationRin        = rin;   }
  void SetIsolationRout(Double_t rout)        { fIsolationRout       = rout;  }
  void SetIsolationTubeHalfH(Double_t tubeH)  { fIsolationTubeHalfH  = tubeH; }
  void SetIsolationCut(Double_t cut)          { fIsolationCut        = cut;   }
  
  void SetChisquareCut(Double_t chi2)         { fChisquareCut        = chi2;  }
  void SetFitterTubeRCut(Double_t tubeR)      { fFitterTubeRCut      = tubeR; }
  void SetFitterTubeHalfHCut(Double_t tubeH)  { fFitterTubeHalfHCut  = tubeH; }
  
  void SetTrackDistanceCut(Double_t dis)      { fTrackDistanceCut    = dis;   }
  
  void SetEHCompareTubeR(Double_t tubeR)      { fEHCompareTubeR      = tubeR; }
  void SetEHCompareTubeHalfH(Double_t tubeH)  { fEHCompareTubeHalfH  = tubeH; }
  void SetEHCompareHERatioCut(Double_t hecut) { fEHCompareHERatioCut = hecut; }
  
  void SetTubeRCut(Double_t tubeR)            { fTubeRCut            = tubeR; }
  void SetTubeHalfHCut(Double_t tubeH)        { fTubeHalfHCut        = tubeH; }
  void SetTOFCut(Double_t tof)                { fTOFCut              = tof; }
  
  void SetMinLayerCut(Double_t minlay)        { fMinLayerCut         = minlay;}
  
 private:
  
 //---------------------------------------------------------
 // for MakeEMClusters()
 //---------------------------------------------------------
  
 // isolation cut
  Double_t fIsolationRin;
  Double_t fIsolationRout;
  Double_t fIsolationTubeHalfH;
  Double_t fIsolationCut;
  
  // chisquare cut
  Double_t fChisquareCut;
  Double_t fFitterTubeRCut;
  Double_t fFitterTubeHalfHCut;
  
  // track distance cut
  Double_t fTrackDistanceCut;
  
  // EHCompapre cut
  Double_t fEHCompareTubeR;
  Double_t fEHCompareTubeHalfH;
  Double_t fEHCompareHERatioCut;
  
  // cut for clustering
  Double_t fTubeRCut;
  Double_t fTubeHalfHCut;
  Double_t fTOFCut;       // Note: the resolution of CAL TOF ~ 1.3 nsec
  
  //---------------------------------------------------------
  // for EMTrackMatch()
  //---------------------------------------------------------
  
  Double_t fMinLayerCut;
  
  
  ClassDef(S4CALEMClusterMaker, 1)

};
    
#endif
    
