//*************************************************************************
//* ---------------------
//* S4CALEMClusterMakerUtils Class
//* ---------------------
//* (Description)
//*
//* (Requires)
//*
//* (Provides)
//*
//* (Update Record)
//*
//*************************************************************************
#include "S4CALEMClusterMakerUtils.h"
#include "S4CALHit.h"
#include "S4CALAddress.h"
#include "TF1.h"
#include "TGraph.h"
#include "TGraphErrors.h"
#include "TVirtualFitter.h"
#include "S4CALClusterUtils.h"

#include "JSFSteer.h"
#include "TNtuple.h"
#include <iostream>


ClassImp(S4CALEMClusterMakerUtils)

#define __THRUST__


  //Int_t    S4CALEMClusterMakerUtils::fNEMLayers = gJSF->Env()->GetValue("J4CAL.EM.NLayers",38);
  //Double_t S4CALEMClusterMakerUtils::fEMTotalRadLength = 27.;
  Int_t    S4CALEMClusterMakerUtils::fNEMLayers = S4CALAddress::GetEMNLayers();
Double_t S4CALEMClusterMakerUtils::fEMTotalRadLength = 26.;
//Int_t    S4CALEMClusterMakerUtils::fNHDLayers = gJSF->Env()->GetValue("J4CAL.HD.NLayers",130);
//Double_t S4CALEMClusterMakerUtils::fHDTotalRadLength = 185.3;
Int_t    S4CALEMClusterMakerUtils::fNHDLayers = S4CALAddress::GetHDNLayers();
Double_t S4CALEMClusterMakerUtils::fHDTotalRadLength = 165.;

//=====================================================================
//* constructor -------------------------------------------------------
S4CALEMClusterMakerUtils::S4CALEMClusterMakerUtils()
  :fMinNHits(10)
{
}

//=====================================================================
//* destructor --------------------------------------------------------

S4CALEMClusterMakerUtils::~S4CALEMClusterMakerUtils()
{
}

//==========================================================================
//* EMFitter ---------------------------------------------------------------

std::vector<Int_t> S4CALEMClusterMakerUtils::EMFitter(std::map<Int_t,S4CALSmallCluster*> &sclmap,
						      S4CALSmallCluster* scls,
						      Double_t tubeR,
						      Double_t tubeH,
						      Double_t E0,
						      Int_t maxitr)
{
  //
  // Need to consider the HCAL signals??
  //

  //
  // The longitudinal profile of the energy deposition in an electromagnetic
  // cascade is described as follows (gamma distribution)
  //
  //   f(t) = dE/dt = E0*b*( ((bt)^(a-1))*exp(-b*t)/gamma(a) )
  //     where t:distance in unit of radiation length, E0:incident energy(GeV)
  //

//#define __FREE__

  std::vector<Int_t> idvec;

  if( (scls->IsLocked()) || !(scls->IsEM()) || (scls->IsMIP())) return idvec;

  //-------------- Initialize ----------------------
  // param[0] = E0, param[1] = a, param[2] = b
  TF1 func = TF1("FUNCINEMFITTER","[0]*[2]*((TMath::Power([2]*x, [1]-1))*exp(-1*[2]*x))/TMath::Gamma([1])", 0., fEMTotalRadLength );

  Double_t radlength[fNEMLayers];     // distance in unit of radiation length
  for(Int_t i = 0; i < fNEMLayers; i++){
    //radlength[i] = (Double_t) fEMTotalRadLength*i/fNEMLayers;
    radlength[i] = (Double_t) fEMTotalRadLength*(i+1.)/fNEMLayers;
  }

  Double_t energy[fNEMLayers];  // e-dep in each layer
  Int_t layerhits[fNEMLayers];  // number of hits in each layer
  Double_t   yerr[fNEMLayers];
  Double_t   xerr[fNEMLayers];
  
  for(Int_t loop = 0; loop < fNEMLayers; loop++){
    energy[loop] = 0.;
    layerhits[loop] = 0;
    yerr[loop] = 0;
    xerr[loop] = 0;
  }

  Double_t coff = 3/300.; //* default
  //Double_t coff = 0.04;    //* thesis

  func.SetParameters(E0*coff, 3.5, 0.5);
  //func.SetParLimits(0, 0., 200.);  // E0
  //func.SetParLimits(1, 1., 6.);    // a
  //func.SetParLimits(2, 0., 2.);    // b
  //------------------------------------------------


  //-------- sum up energy deposit -----------------

  TVector3 motherpos = scls -> GetPosition();

  TVector3 motherthrust = scls -> GetThrustVector();
#ifndef __THRUST__
  motherthrust = motherpos*(1./motherpos.Mag());
#endif

  if(motherthrust.Mag() < 1.e-8) motherthrust = motherpos*(1./motherpos.Mag());

  TVector3 childpos = TVector3(0., 0., 0.);

  std::map<Int_t, S4CALSmallCluster*>::iterator itr;

  Double_t totalEdep = 0.;

  for(itr = sclmap.begin(); itr != sclmap.end(); itr++){

    if( (itr->second->IsLocked()) || !(itr->second->IsEM()) || itr->second->IsMIP()) continue;

    childpos = itr->second->GetPosition();

    if( (motherpos-childpos).Perp(motherthrust) < tubeR
	&& (motherpos-childpos).Mag() < tubeH){

      totalEdep += itr->second->GetEnergyDeposit();

      std::vector<S4CALHit*> calhits = itr->second->GetCALHitsVector();
      for(Int_t hit = 0; hit < (Int_t) calhits.size(); hit++ ){

	Int_t layerID = S4CALAddress::GetLayerID(calhits[hit]->GetCellID(),calhits[hit]->IsEM());
	energy[layerID] += calhits[hit]->GetEnergyDeposit();
	layerhits[layerID]++;
	
	if(layerID > 0){
	  energy[layerID-1] += calhits[hit]->GetEnergyDeposit();
	  layerhits[layerID-1]++;
	}else{  // layer ID = 0
	  energy[layerID]   += calhits[hit]->GetEnergyDeposit();
	  layerhits[layerID]++;
	}
	if(layerID < fNEMLayers-1){ // layer ID != maximum layer ID
	  energy[layerID+1] += calhits[hit]->GetEnergyDeposit();
	  layerhits[layerID+1]++;
	}else{
	  energy[layerID]   += calhits[hit]->GetEnergyDeposit();
	  layerhits[layerID]++;
	}
	
      }
      
    }
  }  

  //------------------------------------------------

  //--- temporary error!! ------------------
  for(Int_t k = 0; k < fNEMLayers; k++){
    if(layerhits[k] > 0){
      yerr[k] = energy[k]/(TMath::Sqrt((Double_t)layerhits[k]));
    }else{
      yerr[k] = 1.;
    }
    //xerr[k] = fEMTotalRadLength/fNEMLayers;
    xerr[k] = 0.;
  }
  //----------------------------------------
  
  //------- Fit -----------
  TGraphErrors graph = TGraphErrors(fNEMLayers, radlength, energy,
				    xerr, yerr); 

  Int_t nitr = TVirtualFitter::GetMaxIterations();

  TVirtualFitter::SetMaxIterations(maxitr);
  graph.Fit(&func, "QN", "", 3., fEMTotalRadLength);
  
  std::vector<Double_t> gammapar;
  gammapar.push_back(func.GetParameter(0)/coff);
  gammapar.push_back(func.GetParameter(1));
  gammapar.push_back(func.GetParameter(2));
  gammapar.push_back(func.GetChisquare()/func.GetNDF());
  gammapar.push_back(TMath::Prob(func.GetChisquare(), func.GetNDF()));
  gammapar.push_back(scls->GetClusterID());  // mother ID
  //-----------------------

  //------ substitute results in small cluster -------
  for(itr = sclmap.begin(); itr != sclmap.end(); itr++){
    
    if(itr->second->IsLocked()) continue;
    if(!itr->second->IsEM()) continue;

    childpos = itr->second->GetPosition();
    if( (motherpos-childpos).Perp(motherthrust) < tubeR
	&& (motherpos-childpos).Mag() < tubeH){   
      //// temporary ////////////
      (itr->second->GetPFOMap()).erase(kgEMFitter);
      ///////////////////////////
      itr->second->AddPFOMap(kgEMFitter, gammapar);
      idvec.push_back(itr->second->GetClusterID());
    }
  }
  //--------------------------------------------------

  TVirtualFitter::SetMaxIterations(nitr);

  return idvec;

}

//==========================================================================
//* EMLongitudinalShape ----------------------------------------------------

std::vector<Int_t> S4CALEMClusterMakerUtils::EMLongitudinalShape(std::map<Int_t,S4CALSmallCluster*> &sclmap, S4CALSmallCluster* scl, Double_t tubeR)
{

  std::vector<Int_t> idvec;

  if( (scl->IsLocked()) || !(scl->IsEM()) || (scl->IsMIP())) return idvec;

  TVector3 pos     = scl->GetPosition();
  TVector3 thrustv = scl->GetThrustVector();
  if(thrustv.Mag() < 1.e-10) thrustv = pos*(1./pos.Mag());
#ifndef __THRUST__
  thrustv = pos*(1./pos.Mag());
#endif

  //---------- calculate mean rho and RMS --------------
  std::map<Int_t,S4CALSmallCluster*>::iterator itr;
  S4CALClusterUtils *util = new S4CALClusterUtils();

  Double_t totale    = 0.;
  Double_t meane     = 0.;
  Double_t meandepth = 0.;
  Double_t cross     = 0.;
  Double_t rms       = 0.;
  Int_t    n         = 0;

  for(itr = sclmap.begin(); itr != sclmap.end(); itr++){

    if(itr->second->IsLocked()) continue;
    if(!(itr->second->IsEM()) || itr->second->IsMIP())  continue;

    if(util->IsWithinTube(itr->second->GetPosition(), pos, thrustv, tubeR, 1.e+10)){

      std::vector<S4CALHit*> hits = itr->second->GetCALHitsVector();
      for(Int_t hit = 0; hit < (Int_t) hits.size(); hit++){
	n++;
	Int_t layerID = S4CALAddress::GetLayerID(hits[hit]->GetCellID(),hits[hit]->IsEM());
	Double_t energy = hits[hit]->GetEnergyDeposit();
	Double_t depth  = fEMTotalRadLength*(layerID+1)/fNEMLayers;
	Double_t eidepthi = energy*depth;
	totale    += energy;
	meandepth += eidepthi;
	cross     += eidepthi*eidepthi;
      }

    }

  }

  meane      = totale/n;
  meandepth /= totale;
  rms = TMath::Sqrt(cross/n - (meane*meandepth)*(meane*meandepth))/meane;
  //----------------------------------------------------

  std::vector<Double_t> paramv;
  paramv.push_back(scl->GetClusterID()); // mother ID
  paramv.push_back(meandepth);
  paramv.push_back(rms);
  paramv.push_back(totale);


  for(itr = sclmap.begin(); itr != sclmap.end(); itr++){

    if(itr->second->IsLocked()) continue;
    if(!(itr->second->IsEM()))  continue;

    if(util->IsWithinTube(itr->second->GetPosition(), pos, thrustv, tubeR, 1.e+10)){
      idvec.push_back(itr->second->GetClusterID());
      ////////////////// temporary /////////////////////
      itr->second->GetPFOMap().erase(kgEMLongitudinal);
      //////////////////////////////////////////////////
      itr->second->AddPFOMap(kgEMLongitudinal,paramv);
    }
  }

  delete util;

  return idvec;

}

//==========================================================================
//* CalcClusterLength ------------------------------------------------------

Double_t S4CALEMClusterMakerUtils::CalcClusterLength(S4CALRealCluster *cls, Double_t elimit)
{

  static Double_t tubeH    = 20.; // half height of tube
  static const Int_t nstep = 100;
  static Double_t step     = tubeH/nstep;

  Double_t length = 0.;

  Double_t etotal[2*nstep] = {0.};

  TVector3 thrust = cls->GetThrustVector();
#ifndef __THRUST__
  thrust = (cls->GetPosition())*(1./(cls->GetPosition()).Mag());
#endif

  std::vector<S4CALHit*> hitv = cls->GetCALHitsVector();

  //------- loop over all hits in a cluster ----------
  for(Int_t i = 0; i < (Int_t) hitv.size(); i++){

    TVector3 hitpos = hitv[i]->GetCellPosition();
    hitpos = hitpos - cls->GetPosition();

    //////////////////////////////////////////////////////////
    // Note.                                                //
    // if v1:(unit vector), pos:(position vector) then      //
    //    k = v1*pos  <==>  v1*(pos - k*v1) = 0.            //
    // where "*" means "(inner) product"                    //
    // (namely, pos-k*v1 vector perpendicular to v1 vector) //
    //////////////////////////////////////////////////////////

    Double_t kperp = hitpos*thrust;
    Double_t     k = 0.;
    Int_t      bin = 0;
    
    for(k = -1.*tubeH; k < tubeH; k+= step){
      if( (kperp < k+step) && (kperp > k) ){
	etotal[bin] += hitv[i]->GetEnergyDeposit();
	break;
      }
      bin++;
    }

  }
  //------------------------------------------------

  Double_t esum = 0.;

  for(Int_t i = 0; i < 2*nstep; i++){
    esum += etotal[i];
    if(esum > 0.) length += step;
    if(esum/(cls->GetEnergyDeposit()) > elimit) break;
  }

  return length;

}

//==========================================================================
//* EMLateralSpread --------------------------------------------------------

std::vector<Int_t> S4CALEMClusterMakerUtils::EMLateralSpread(std::map<Int_t,S4CALSmallCluster*> &sclmap, S4CALSmallCluster* scls, Double_t multiplein, Double_t multipleout, Double_t tubeH)
{

  std::vector<Int_t> idvec;

  if( !(scls->IsEM()) || (scls->IsLocked()) || (scls->IsMIP()) ) return idvec;

  //------ rough calculation! ---------
  //  static const Double_t actth = gJSF->Env()->GetValue("J4CAL.EM.ActiveLayerThickness",0.1);
  //  static const Double_t absth = gJSF->Env()->GetValue("J4CAL.EM.AbsLayerThickness",0.4);
  static const Double_t actth = S4CALAddress::GetEMActiveLayerThickness();
  static const Double_t absth = S4CALAddress::GetEMAbsLayerThickness();
  Double_t X0 = fNEMLayers*(actth+absth+0.1)/fEMTotalRadLength;

  Double_t Z  = 82.;
  Double_t Es = 21.2052;         // scale energy (MeV)
  Double_t Ec = 800./(Z + 1.2);  // critical energy in Rossi definition

  Double_t moliereR = X0*Es/Ec;  // Moliere radius(cm)
  //-----------------------------------

  TVector3 pos     = scls->GetPosition();
  TVector3 thrustv = scls->GetThrustVector();
  if(thrustv.Mag() < 1.e-10) thrustv = pos*(1./pos.Mag());
#ifndef __THRUST__
  thrustv = pos*(1./pos.Mag());
#endif

  std::map<Int_t,S4CALSmallCluster*>::iterator itr;

  S4CALClusterUtils *util = new S4CALClusterUtils();

  //---------- calculate energy ratio ----------------
  Double_t Ein  = 0.;
  Double_t Eout = 0.;
  for(itr = sclmap.begin(); itr != sclmap.end(); itr++){

    if(itr->second->IsLocked() || itr->second->IsMIP()) continue;

    if(util->IsWithinTube(itr->second->GetPosition(),pos,thrustv,moliereR*multipleout,tubeH)){

      std::vector<S4CALHit*> hitv = itr->second->GetCALHitsVector();

      for(Int_t k = 0; k < (Int_t) hitv.size(); k++){
	TVector3 hpos = hitv[k]->GetCellPosition();
	if( ((hpos-pos).Perp(thrustv) < moliereR*multiplein) && ((hpos-pos).Mag() < tubeH) ){
	  Ein  += hitv[k]->GetEnergyDeposit();
	}else{
	  Eout += hitv[k]->GetEnergyDeposit();
	}
      }

    }

  }

  Double_t ratio = Eout/Ein;
  //--------------------------------------------------

  //------------ push_back parameters ----------------
  std::vector<Double_t> paramv;
  paramv.push_back(scls->GetClusterID()); // 0
  paramv.push_back(ratio);                     // 1
  paramv.push_back(moliereR);                  // 2
  paramv.push_back(multiplein);                // 3
  paramv.push_back(multipleout);               // 4
  //--------------------------------------------------


  //----- insert parameters into small custers -------
  for(itr = sclmap.begin(); itr != sclmap.end(); itr++){

    if(itr->second->IsLocked()) continue;

    if(util->IsWithinTube(itr->second->GetPosition(),pos,thrustv,moliereR*multiplein,tubeH)){
      idvec.push_back(itr->second->GetClusterID());
      ////////////////// temporary /////////////////////
      itr->second->GetPFOMap().erase(kgEMLateralSpread);
      //////////////////////////////////////////////////
      itr->second->AddPFOMap(kgEMLateralSpread,paramv);
    }
  }
  //--------------------------------------------------

  delete util;

  return idvec;
  
}

//==========================================================================
//* EECompare --------------------------------------------------------------

std::vector<Int_t> S4CALEMClusterMakerUtils::EECompare(std::map<Int_t,S4CALSmallCluster*> &sclmap,
						       S4CALSmallCluster* mscl,
						       Double_t tubeR)
{
  std::vector<Int_t> idvec;
  std::map<Int_t,S4CALSmallCluster*>::iterator itr;

  if(!(mscl->IsEM())) return idvec;
  if(mscl->IsLocked() || mscl->IsMIP()) return idvec;

  TVector3 mpos = mscl -> GetPosition();
  TVector3 mthrust = mscl -> GetThrustVector();

  //if(mthrust.Mag() < 1.e-15) return idvec;
  if(mthrust.Mag() < 1.e-15) mthrust = mpos*(1./mpos.Mag());
#ifndef __THRUST__
  mthrust = mpos*(1./mpos.Mag());
#endif

  Int_t    mID            = mscl->GetClusterID();
  Double_t EnergyDeposit  = 0.;   // e-dep

  //---- sum up energy within a tube -----------
  TVector3 childpos = TVector3(0., 0., 0.);
  for(itr = sclmap.begin(); itr != sclmap.end(); itr++){

    if(itr->second->IsLocked() || itr->second->IsMIP()) continue;

    childpos = itr->second->GetPosition();

    if( (mpos-childpos).Perp(mthrust) < tubeR && itr->second->IsEM()){
      EnergyDeposit += itr->second->GetEnergyDeposit();
    }

  }
  //--------------------------------------------
  
  //--- set sampling tube radius and boundary (temporary) ---
  //  static const Int_t    emnlayers = gJSF->Env()->GetValue("J4CAL.EM.NLayers",38);
  //  static const Double_t absth  = gJSF->Env()->GetValue("J4CAL.EM.AbsLayerThickness",0.4);
  //  static const Double_t scith  = gJSF->Env()->GetValue("J4CAL.EM.ActiveLayerThickness",0.1);

  //  static const Double_t emrin  = gJSF->Env()->GetValue("J4CAL.BarrelTower.FrontRadius",210.);
  static const Int_t emnlayers = S4CALAddress::GetEMNLayers();
  static const Double_t scith  = S4CALAddress::GetEMActiveLayerThickness();
  static const Double_t absth  = S4CALAddress::GetEMAbsLayerThickness();
  static const Double_t emrin  = S4CALAddress::GetBarrelInnerR();
  static const Double_t emrout = emrin + emnlayers*(absth+scith+0.1);

  Double_t boundary = 0.;
#if 1
  boundary = 0.5*(EnergyDeposit*51.) + 11.5;
#else
  boundary = (emrout - emrin)/2.;
#endif

  if( boundary > (0.9*(emrout - emrin) + emrin) )
    boundary = (0.9*(emrout - emrin) + emrin);

  boundary *= fEMTotalRadLength/19.8;  // cm -> radiaion length
  //---------------------------------------------------------


  //-------- calculate total e-dep within tube --------------
  Double_t EnergyU = 0.;   // upper
  Double_t EnergyL = 0.;   // lower

  for(itr = sclmap.begin(); itr != sclmap.end(); itr++){

    if(itr->second->IsLocked() || itr->second->IsMIP()) continue;
    childpos = itr->second->GetPosition();
    if( (mpos-childpos).Perp(mthrust) < tubeR && itr->second->IsEM()){
      std::vector<S4CALHit*> hits = itr->second->GetCALHitsVector();
      for(Int_t hit = 0; hit < (Int_t) hits.size(); hit++){
	TVector3 cellpos = hits[hit]->GetCellPosition();
	Int_t layerID = S4CALAddress::GetLayerID(hits[hit]->GetCellID(),
						 hits[hit]->IsEM());
	Double_t depth = fEMTotalRadLength*(layerID+1)/fNEMLayers;
	(depth < boundary) ?
	  (EnergyL += itr->second->GetEnergy()):
	  (EnergyU += itr->second->GetEnergy());
      }
    }
  }

  //------------------------------------------
  Double_t ERatio = EnergyU/(EnergyU + EnergyL);
  std::vector<Double_t> emvector;
  emvector.push_back(mID);
  emvector.push_back(ERatio);
  emvector.push_back(tubeR);

  for(itr = sclmap.begin(); itr != sclmap.end(); itr++){

    childpos = itr->second->GetPosition();
    if( (mpos-childpos).Perp(mthrust) < tubeR && itr->second->IsEM()){
      /// temporary //////////////////////////////
      (itr->second->GetPFOMap()).erase(kgEECompare);
      ////////////////////////////////////////////
      itr->second->AddPFOMap(kgEECompare, emvector);
      idvec.push_back(itr->second->GetClusterID());
    }
  }  // end of for loop
  //------------------------------------------

  return idvec;

}

//==========================================================================
//* EHCompare --------------------------------------------------------------

//
// EHCompare() compare the energy-deposit between EM/HD Calorimeter within a
// tube. 
//

std::vector<Int_t> S4CALEMClusterMakerUtils::EHCompare(std::map<Int_t,S4CALSmallCluster*> &sclmap,
						       S4CALSmallCluster* mscl,
						       Double_t tubeR,
						       Double_t tubeH)
{

  std::vector<Int_t> idvec;

  if(!(mscl->IsEM())) return idvec;
  if(mscl->IsLocked()) return idvec;

  TVector3 mpos = mscl -> GetPosition();
  TVector3 mthrust = mscl -> GetThrustVector();

  //if(mthrust.Mag() < 1.e-15) return idvec;
  if(mthrust.Mag() < 1.e-15) mthrust = mpos*(1./mpos.Mag());
#ifndef __THRUST__
  mthrust = mpos*(1./mpos.Mag());
#endif

  typedef std::multimap<MatchedType_t,std::vector<Double_t> > MATCH;
  std::pair<MATCH::iterator,MATCH::iterator> itr_range;
  MATCH::iterator mitr;
  std::map<Int_t,S4CALSmallCluster*>::iterator itr;

  for(itr = sclmap.begin(); itr != sclmap.end(); itr++){
    itr_range = (itr->second->GetPFOMap()).equal_range(kgEHCompare);
    for(mitr = itr_range.first; mitr != itr_range.second; mitr++){
      if((mitr->second)[0] == mscl->GetClusterID()){
	(itr->second->GetPFOMap()).erase(mitr);
	break;
      }
    }
  }

  Int_t mID  = mscl->GetClusterID();
  Double_t EMEnergy = 0.;
  Double_t HDEnergy = 0.;

  //---- check hits within a tube -----------
  EMEnergy = 0.;
  HDEnergy = 0.;

  TVector3 childpos = TVector3(0., 0., 0.);
  for(itr = sclmap.begin(); itr != sclmap.end(); itr++){

    if(itr->second->IsLocked()) continue;
    childpos = itr->second->GetPosition();
    if( (mpos-childpos).Perp(mthrust) < tubeR && (mpos-childpos).Mag() < tubeH){
      itr->second->IsEM() ?
	EMEnergy += itr->second->GetEnergy():
	HDEnergy += itr->second->GetEnergy();
    }

  }

  Double_t ERatio = HDEnergy/EMEnergy;
  //------------------------------------------

  //----- subtract parameters into scls ------
  std::vector<Double_t> emvector;
  emvector.push_back(mID);
  emvector.push_back(ERatio);   // temporary
  emvector.push_back(tubeR);
  emvector.push_back(tubeH);
  
  for(itr = sclmap.begin(); itr != sclmap.end(); itr++){

    if(itr->second->IsLocked()) continue;
    
    childpos = itr->second->GetPosition();
    if( (mpos-childpos).Perp(mthrust) < tubeR && (mpos-childpos).Mag() < tubeH ){
      /// temporary //////////////////////////////
      (itr->second->GetPFOMap()).erase(kgEHCompare);
      ////////////////////////////////////////////
      itr->second->AddPFOMap(kgEHCompare, emvector);
      idvec.push_back(itr->second->GetClusterID());
    }
  }  // end of for loop
  //------------------------------------------

  return idvec;

}
