#ifndef __S4CDCANALYSIS__
#define __S4CDCANALYSIS__

//*************************************************************************
//* -----------
//* S4CDCAnalysis
//* -----------
//* (Description)
//* 	Clasess to analyze CDC data.
//* (Update Record)
//*	2000/05/16  K.Fujii	Original version.
//*************************************************************************

#include "TObject.h"
#include "TH2.h"
#include "TH1E.h"
#include "TCanvas.h"

#include "S4CDCHitMaker.h"


class S4CDCAnalysis : public JSFModule {
public:
   S4CDCAnalysis(const Char_t *name="S4CDCAnalysis",
		  const Char_t *title="CDC Analysis");
   virtual ~S4CDCAnalysis();

   inline void  SetPtCut(Double_t pt) { fPtcut = pt; }
   inline void  SaturationOff() { fSaturation = kFALSE; }
   inline void  SaturationOn()  { fSaturation = kTRUE; }

   Bool_t Initialize();
   Bool_t Process(Int_t ev);
   
   void Draw(Option_t *option="");

private:
   virtual void MonitorHits(Int_t ev);
   virtual void MonitorTracks(Int_t ev);
   virtual void MonitorMotherTrackNo(Int_t ev);
   
private:
   struct THdef1 {
      Char_t   title[256];
      Int_t    nxbin;
      Float_t  xmin;
      Float_t  xmax;
   };
   struct THdef2 {
      Char_t   title[256];
      Int_t    nxbin;
      Float_t  xmin;
      Float_t  xmax;
      Int_t    nybin;
      Float_t  ymin;
      Float_t  ymax;
   };
   
   Double_t fPtcut;		// Pt cut value
   
   TH2D	   **hRDX;		//!
   TH2D	   **hRDXmin;		//!
   TH2D	   **hEff;		//!
   TH2D	   **hPTvsRes;		//!
   TH2D	   **hCTvsRes;		//!
   TH2D	   **hFIvsRes;		//!
   TH1E	   **hREff;		//!
   TH1D	   **hEff1;		//!
   TH1D	   **hNHits;		//!

   TH1D     *hChi2;             //!
   TH1D     *hNdf;              //!
   TH1D     *hCL;               //!
   TH1D     *hT0;               //!
   TH2D     *hNdfT0;            //!

   TCanvas  *hcRDX;		//! canvas to draw residual plots
   TCanvas  *hcRDXmin;		//! canvas to draw residual plots
   TCanvas  *hcREff;		//! canvas to draw residual plots
   TCanvas  *hcPTvsRes;		//! canvas to draw residual plots
   TCanvas  *hcCTvsRes;		//! canvas to draw residual plots
   TCanvas  *hcFIvsRes;		//! canvas to draw residual plots
   TCanvas  *hcEff1;		//! canvas to draw residual plots
   TCanvas  *hcNHits;		//! canvas to draw residual plots
   THdef2   *hdefRDX;		//! histogram parameters for hRes
   THdef2   *hdefRDXmin;	//! histogram parameters for hRes
   THdef2   *hdefPTvsRes;	//! histogram parameters for hRes
   THdef2   *hdefCTvsRes;	//! histogram parameters for hRes
   THdef2   *hdefFIvsRes;	//! histogram parameters for hRes
   THdef1   *hdefREff;		//! histogram parameters for hRes
   THdef1   *hdefEff1;		//! 
   THdef1   *hdefNHits;		//! 
   

   static const Int_t kNh;	//! # hists
   static const Int_t kZoneX;	//! # X divisions
   static const Int_t kZoneY;	//! # Y divisions
   
   Bool_t    fSaturation;	// artificial saturation flag

   ClassDef(S4CDCAnalysis, 1) // CDC Analysis class
};

#endif


