//*****************************************************************************
//* ==============
//* class S4VTrack
//* ==============
//* (Purpose)
//*    Attribute class for METIS (smeared) track.
//* (Dependency)
//*    Requires: 
//*    Provides: S4VTrack
//*
//* (Update Record)
//*   2004/09/05  K.Fujii   Orivinal very primitive version.
//*****************************************************************************

#include <iostream>
#include <iomanip>

#include "JSFHelicalTrack.h"

#include "U4VTrack.h"
#include "S4VTrack.h"
#if __GNUC__ >= 5 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3)
#include <cstdlib>
#endif

ClassImp(S4VTrack)

S4VTrack::ESortOption_t S4VTrack::fgSortBy = S4VTrack::kTrackID;

//=============================================================================
//* C-tors and D-tor ----------------------------------------------------------

S4VTrack::S4VTrack()
         : fT (0.),
           fXv(0.,0.,0.),
           fE (0.),
           fPv(0.,0.,0.),
           fDphiLast(-1.e10),
           fCharge(0.)
{
}

S4VTrack::~S4VTrack()
{
}

//=============================================================================
//* Getters -------------------------------------------------------------------

Double_t S4VTrack::GetTime(Double_t d)
{
   if (d == fDphiLast) return fT;
   fDphiLast = d;
   CalcStateAt(d);
   return fT;
}

TVector3 S4VTrack::GetPosition(Double_t d)
{
   if (d == fDphiLast) return fXv;
   fDphiLast = d;
   CalcStateAt(d);
   return fXv;
}

Double_t S4VTrack::GetEnergy(Double_t d)
{
   if (d == fDphiLast) return fE;
   fDphiLast = d;
   CalcStateAt(d);
   return fE;
}


TVector3 S4VTrack::GetMomentum(Double_t d)
{
   if (d == fDphiLast) return fPv;
   fDphiLast = d;
   CalcStateAt(d);
   return fPv;
}

Double_t S4VTrack::GetCharge()
{
   using namespace std;

   if (fCharge == 0.) {
      U4VTrack *vtrkp = dynamic_cast<U4VTrack *>(this);
      if (!vtrkp) {
         cerr << ">>>>>> S4VTrack::CalcStateAt >>>>>>>>>>>>>>>>>>>>>>>>>" << endl
              << "  Can't to dynamic_cast this to <U4VTrack *>, though  " << endl
              << "  S4VTrack is supposed to be inherited together with  " << endl
              << "  U4VTrack!    Abort!"                                  << endl;
         abort();
      } else {
         JSFHelicalTrack *help = vtrkp->GetHelicalTrackPtr();
         fCharge = (help ? help->GetCharge() : -99999.);
      }
   }
   return fCharge;
}

//=============================================================================
//* CalcStateAt ---------------------------------------------------------------

void S4VTrack::CalcStateAt(Double_t dphi)
{
   using namespace std;
   U4VTrack *vtrkp = dynamic_cast<U4VTrack *>(this);

   if (!vtrkp) {
      cerr << ">>>>>> S4VTrack::CalcStateAt >>>>>>>>>>>>>>>>>>>>>>>>>" << endl
           << "  Can't to dynamic_cast this to <U4VTrack *>, though  " << endl
           << "  S4VTrack is supposed to be inherited together with  " << endl
           << "  U4VTrack!    Abort!"                                  << endl;
      abort();
   } else {
      JSFHelicalTrack *help = vtrkp->GetHelicalTrackPtr();
      if (help) {
#if 0
         help->MovePivot(JSF3DV(0.,0.,0.));   // move pivot to origin
#else
         dphi += help->GetDeflectionAngle2D(JSF2DV(0.,0.));   // get deflection angle
#endif
         JSF3DV p = help->GetMomentum(dphi);
         fPv      = TVector3(p.X(), p.Y(), p.Z());
         fE       = fPv.Mag();                      // ignore mass
         JSF3DV x = help->GetCoordinate(dphi);
         fXv      = TVector3(x.X(), x.Y(), x.Z());
         fT       = vtrkp->GetT0();                 // ignore TOF
      }
   }
}
