#ifndef __S4TPCHIT__
#define __S4TPCHIT__
//*************************************************************************
//* ---------------------
//* S4TPCHit Class
//* ---------------------
//* (Description)
//*	Class for a single TPC hit.
//* (Requires)
//*	TObject
//* (Provides)
//*	S4TPCHit		: TObject containing single hit data
//* (Update Record)
//*	2004/09/06  A.Yamaguchi	Original version.
//*************************************************************************

#include "S4TPCExactHit.h"
#include "U4TPCHit.h"
#if __GNUC__ >= 5 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3)
#include <cstdlib>
#endif

class S4TPCHit : public U4TPCHit {
public:
  S4TPCHit(S4TPCExactHit * exacthit);
  
  //==========================================================
  // --- ATTENTION ------
  // Both copy constructor and operator= don't set "this" pointer
  // to their ExactHit's Smeared Hit Pointer.
  //
     inline       S4TPCHit(const S4TPCHit & right);
     inline const S4TPCHit& operator=(const S4TPCHit& right);
  //==========================================================
  
  virtual ~S4TPCHit();

  // Getter --------------------------------------------------------------- 
  inline S4TPCExactHit *GetExactHitPtr() const { return fExactHitPtr;       }
  
  // Setter --------------------------------------------------------------- 
  static void  SetSpatialResolution(TString opt) { 
     fgSpatialResolutionCurve = opt;
  }
  static void  SetSortBy(Char_t *opt = "TrackID");
  static void  SetSigmaZ   (Double_t sg = 0.) { fgDZdrift  = sg; }
  static void  SetSigmaRPhi(Double_t sg = 0.) { fgDRPhi    = sg; }
  static void  SetT0offset (Double_t t0 = 0.) { fgT0offset = t0; }

  static void SetSigma0(Double_t sg=0.0) { fgSigma0 = sg; }
  static void SetCd    (Double_t sg=0.0) { fgCd = sg; }
  static void SetAlpha (Double_t sg=0.0) { fgAlpha = sg; }
  static void SetNe    (Double_t sg=0.0) { fgNe = sg; }

  static void SetSig00      (Double_t sg=0.0) { fgSig00 = sg; }
  static void SetSig0SinDep (Double_t sg=0.0) { fgSig0SinDep = sg; }
  static void SetSigLDep    (Double_t sg=0.0) { fgSigLDep = sg; }
  static void SetSigDZLDep    (Double_t sg=0.0) { fgSigDZLDep = sg; }

  // other method --------------------------------------------------------- 
  inline Bool_t  IsSortable() const { return kTRUE;            }

  virtual Int_t  Compare(const TObject *obj) const;
  void           DebugPrint(const Char_t * opt = "Brief");
  static void    Print();

private:
  virtual void   Update();
  virtual void   CalculatePosition();

private:
  static Double_t  fgDZdrift;                  // zdrift resolution [cm]
  static Double_t  fgDRPhi;                    // rphi resolution [cm]
  static Double_t  fgDVdrift;                  // drift velocity [cm/micro sec]
  static Double_t  fgDT0;                      // t0 resolution [cm]


  static Double_t  fgSigma0;  // Constant term (sig0) for experimental formula [cm]
  static Double_t  fgCd; // Cd fir experimental formula [cm/cm^{0.5}]
  static Double_t  fgAlpha; // alpha term for experimental formula 
  static Double_t  fgNe;    // Ne term for experimental formula

  static Double_t  fgSig00; // Constant term for ILD formula [cm]
  static Double_t  fgSig0SinDep; // Sin dependant sig0 for ILD formula [cm]
  static Double_t  fgSigLDep; // Drift length dependant term for ILD formula[cm/cm^{0.5}]
  static Double_t  fgSigDZLDep; // Drift length dependant term for ILD formula[cm/cm^{0.5}]

  static TString   fgSpatialResolutionCurve;   // "flat" or "experimental"
  static Int_t     fgS4TPCHitSortBy;           // sort flag
  static Double_t  fgT0offset;                 // T0 [nsec]
  S4TPCExactHit   *fExactHitPtr;               // Pointer to Exact Hit 

  ClassDef(S4TPCHit, 1)                        // TPC Anode Hit
};

//=====================================================================
//---------------------------------
// inline function for S4TPCHit
//---------------------------------

inline S4TPCHit::S4TPCHit(const S4TPCHit& right)
{
   fExactHitPtr        = right.fExactHitPtr;
   abort();   
   // !!! Don't set this to S4TPCExactHit::SmearedHitPointer !!! 
}

inline const S4TPCHit&
             S4TPCHit::operator= (const S4TPCHit& right)
{            
   fExactHitPtr        = right.fExactHitPtr;
   
   abort();   
   // !!! Don't set this to S4TPCExactHit::SmearedHitPointer !!! 
                  
     return *this;
}

#endif
