#include "UserAnalysis.h"
#include <sstream>

ClassImp(UserAnalysis)

const Int_t     kNmods         = 10;
const Int_t     kFullDriftTime = 6492;
const Int_t     kNhodos        = __HODOX__ + __HODOY__;
const Double_t  kHodThresh     = 30.;

#if 1
const Double_t  kHodPed[] = {110., 110., 103., 107.,   102., 129., 117., 125.,
                             121., 113., 120., 118.,   100.,  99.,  97.,  95.,
                             105.,  82.,  89., 100.,   105.,  98., 104., 136.,
                             109., 109., 111., 102.,   104., 107., 111., 116.};
#else
const Double_t  kHodPed[] = {135., 137., 133., 134.,   134., 134., 142., 143.,
                             140., 137., 144., 145.,   131., 129., 133., 129.,
                             131., 111., 122., 127.,   131., 134., 132., 129.,
                             132., 137., 142., 130.,   139., 135., 139., 134.};
#endif



//=================================================================
//----------------
//  constructor 

UserAnalysis::UserAnalysis(BabyDisplay *display = 0,
		           		   U4CDCFADCCalibration *calib = 0) :
		      VUserAnalysis(display,calib),
		      fLabel(0.007,0.96,0.20,0.99,"Counter Info","br")
{
    BabyDisplay::SetNpages(4);
}
	
//=================================================================
//----------------
//  destructor 
	
UserAnalysis::~UserAnalysis() 
{
   for (Int_t i = 0; i < kNmods; i++ ) {
   	if (hFADC[i]) delete hFADC[i];
   }

   for (Int_t i = 0; i < kNhodos; i++ ) {
   	if (hHADC[i]) delete hHADC[i];
   }

   if (hFADC)     	delete [] hFADC; 
   if (hHADC)   	delete [] hHADC; 
   if (hHod1)		delete hHod1;
   if (hHod2)		delete hHod2;
   if (hHod12)		delete hHod12;
   if (hHodADC12)	delete hHodADC12;
   if (hHod12lo)	delete hHod12lo;
   if (hHod12hi)	delete hHod12hi;
   if (hTC1TC2)		delete hTC1TC2;
   if (hMonHod)		delete hMonHod;
   if (hHod1hs)		delete hHod1hs;
   if (hNHod12)		delete hNHod12;
   if (hMonCsi)		delete hMonCsi;
}


//=================================================================
//-------------------------------
void UserAnalysis::InitRawFADC()
{
	
	cerr << " ************************************ " << endl;
	cerr << " ***       !!! ATTENTION !!!      *** " << endl;
	cerr << " *** This version of UserAnalysis *** " << endl;
	cerr << " ***     requires FormatID = 1.   *** " << endl;
	cerr << " ************************************ " << endl;
		
	
  TDirectory *last = gDirectory;
  gJSF->GetOutput()->cd("/");
  Char_t st[256];
  
  hFADC  = new TH1F*[kNmods];
 
  for (Int_t i = 0; i < kNmods; i++) {
     sprintf(st,"hFADC%2.2d",i);
     if (i < 8) hFADC[i] = new TH1F(st,"",2200,0.,6600.);
     else	hFADC[i] = new TH1F(st,"",3300,0.,6600.);
  }
  last->cd();
}

//=================================================================
//-------------------------------
void UserAnalysis::DrawRawFADC()
{
  TDirectory *last = gDirectory;
  gFile->cd("/");

  U4BabyTrackMaker    *tm = (U4BabyTrackMaker*)gJSF->FindModule("U4BabyTrackMaker");
  U4BabyTrackMakerBuf *tb = (U4BabyTrackMakerBuf *)tm->EventBuf();
  TObjArray           *ta = tb->GetTracks();

  U4BabyTrack *tp;
  TIter nexttrack(ta);
    
  if (!(GetDisplay()->IsInLoop())) {

    TCanvas *da = (TCanvas *)gROOT->FindObject("DisplayArea");  
    da->cd();
    
    Int_t page = GetDisplay()->GetPage();
    switch (page) {
      case 0: {

        U4CDCRawData     *raw = (U4CDCRawData*)gJSF->FindModule("U4CDCRawData");
        U4CDCRawDataBuf  *evt = (U4CDCRawDataBuf *)raw->EventBuf();
        U4CDCBeginRunBuf *br  = raw->GetBeginRunBuf();
        U4CDCHitMaker    *hm  = (U4CDCHitMaker*)gJSF->FindModule("U4CDCHitMaker");
        U4CDCHitMakerBuf *hb = (U4CDCHitMakerBuf *)hm->EventBuf();
        TObjArray        *la = hb->GetOutputBufPtr();

        Int_t              ncdc = evt->GetNADCData();
        TClonesArray   *adcdata = evt->GetADCData();

        for (Int_t i = 0; i < kNmods; i++) {
          hFADC[i]->Reset();
        }

        Int_t npres = __NPRES__ ;

        for (Int_t i = 0; i < ncdc; i++) {
          U4CDCRawADCDatum *adcdatum = (U4CDCRawADCDatum *)adcdata->UncheckedAt(i);
          Int_t a;
          Int_t id;
          Int_t cr = 0;                    // We have only 1 crate now.
          Int_t st = adcdatum->GetStationNo();
          Int_t ch = adcdatum->GetChannelNo();
          
          if (st > __CHAMBERSTLAST__) break; 
          
          br->EChannelToDChannel(cr,st,ch,id,a);

          if (a > 999999) break;

          Double_t ped  = GetCalib()->GetPedestal(st,ch);
          Double_t gain = GetCalib()->GetGain(st,ch);      
          Double_t tped = GetCalib()->GetTimePedestal(st,ch) + npres;
          Double_t tskl = GetCalib()->GetTimeScale(st,ch);
          Double_t adc  = (ped - adcdatum->GetADC())*gain;
          Double_t time = (adcdatum->GetTime() - tped)*tskl;
          if (a >= __NWIRESPERCELL__) time = kFullDriftTime - time;
          if ( id == 0 && a >= 0 && a < kNmods ) {
            hFADC[a]->Fill(time,adc);
          }
        }

    	da->Divide(1,kNmods);
    	for (Int_t i = 0; i < kNmods; i++) {
      	  da->cd(i+1);
      	  gStyle->SetOptStat(0);
      	  gPad->SetTopMargin(0.01);
      	  gPad->SetBottomMargin(0.2);
      	  hFADC[i]->GetXaxis()->SetTickLength(0.1);
      	  hFADC[i]->GetYaxis()->SetTickLength(0.005);
      	  hFADC[i]->GetXaxis()->SetNdivisions(30);
      	  hFADC[i]->GetYaxis()->SetNdivisions(4); 
      	  hFADC[i]->GetXaxis()->SetLabelSize(0.20); 
      	  hFADC[i]->GetYaxis()->SetLabelSize(0.20); 
      	  hFADC[i]->Draw();
    	}
        da->Update();

#if 1	// write markers on the histgram

    	U4CDCHitLayer *layerp;
    	TIter nextLayer(la);
    	while ((layerp = (U4CDCHitLayer *)nextLayer())) {
      	  TIter nextCell(layerp);
      	  U4CDCHitCell *cellp;
      	  while ((cellp = (U4CDCHitCell *)nextCell())) {
	    TIter nextWire(cellp);
	    U4CDCHitWire *wirep;
	    while ((wirep = (U4CDCHitWire *)nextWire())) {
	      Int_t layerNo = wirep->GetLayerNumber();
	      Int_t cellNo  = wirep->GetCellNumber();
	      Int_t wireNo  = wirep->GetWireNumber();
	      Int_t a       = __NWIRESPERCELL__ *layerNo + wireNo; 
	      if (cellNo == 1) a = 10 + wireNo;    // works only for baby
	      if (a < 0 || a > 9) continue;        // works only for baby
	      Char_t str[256];
	      sprintf(str,"hHit%2.2d",a);
	      DeleteGraph(str);
	      TGraph *gr = new TGraph();
	      gr->SetName(str);
	      da->cd(a+1);
	      U4CDCHit *hitp;
	      TIter nextHit(wirep);
	      Int_t nhits = 0;
	      while ((hitp = (U4CDCHit *)nextHit())) {
	        Double_t time = hitp->GetClusterPtr()->GetStartTime();
	        if (a >= 5) time = kFullDriftTime - time;
	        gr->SetPoint(nhits++,time,0.);
	        gr->SetFillColor(19);
	        gr->SetMarkerColor(1);
	        gr->SetMarkerStyle(23);
	        gr->SetMarkerSize(1.5);
	        gr->Draw("P");
	      }
	    }
          }
        }

        nexttrack.Reset();
        Int_t ntrks = 0;
        Int_t ntrkhits = 0;
        while ((tp = (U4BabyTrack *)nexttrack())) {
	  tp->DebugPrint();
          ntrks++;
          U4CDCTrackHit *trkhitp;
          U4CDCHit *hitp;
          TIter nexttrkhit(tp);
          while ((trkhitp = (U4CDCTrackHit *)nexttrkhit())) {
	    hitp = trkhitp->GetHitPtr();
	    hitp->DebugPrint();
	    Int_t layerNo = hitp->GetLayerNumber();
	    Int_t cellNo  = hitp->GetCellNumber();
	    Int_t wireNo  = hitp->GetWireNumber();
	    Int_t a       = 5*layerNo + wireNo;  // works only for baby
	    if (cellNo == 1) a = 10 + wireNo;    // works only for baby
	    if (a < 0 || a > 9) continue;        // works only for baby
	    Double_t time = hitp->GetClusterPtr()->GetStartTime();
	    if (a >= 5) time = kFullDriftTime - time;
	    da->cd(a+1);
	    Char_t str[256];
	    sprintf(str,"hTrackHit%2.2d%3.3d",a,ntrkhits++);
	    DeleteGraph(str);
	    TGraph *gr = new TGraph();
	    gr->SetName(str);
	    gr->SetPoint(0,time,0.);
	    gr->SetFillColor(19);
	    gr->SetMarkerColor((1+ntrks)%50);
	    gr->SetMarkerStyle(8);
	    gr->SetMarkerSize(0.8);
	    gr->Draw("P");
          }
        }
    	break;
#endif
      }
    }
    da->cd();
    da->Update();
  }
  last->cd();
}

//=================================================================
//-------------------------------
void UserAnalysis::InitCounters()
{
  TDirectory *last = gDirectory;
  gJSF->GetOutput()->cd("/");

  hHADC  = new TH1F*[kNhodos];
 
  Char_t st[256];
  for (Int_t i = 0; i < kNhodos-1; i++) {
     sprintf(st,"hHADC%2.2d",i);
     hHADC[i] = new TH1F(st,"",128,0.,2048.);
  }
  hHod1     = new TH1F("hHod1","Hodoscope 1",16,0.,16.);
  hHod2     = new TH1F("hHod2","Hodoscope 2",16,0.,16.);
  hHod12    = new TH2F("hHod12","Hod1 v.s. Hod2",16,0.,16.,16,0.,16.);
  hHodADC12 = new TH2F("hHodADC12","SumHod1 v.s. SumHod2",
                        128,0.,8192.,128,0.,8192.);
  hHod12lo = new TH2F("hHodADC12lo","SumHod1 v.s. SumHod2 (single)",
                        128,0.,8192.,128,0.,8192.);
  hHod12hi = new TH2F("hHodADC12hi","SumHod1 v.s. SumHod2 (double)",
                        128,0.,8192.,128,0.,8192.);
  hTC1TC2   = new TH2F("hTC1TC2","TC1 v.s. TC2",
                        128,0.,2048.,128,0.,2048.);
  hMonHod   = new TH2F("hMonHod","Mon v.s. Hod",
                        128,0.,2048.,128,0.,16384.);
  hHod1hs   = new TH2F("hHod1hs","Hod1: Hard v.s. Soft",
                        128,0.,2048.,128,0.,16384.);
  hNHod12   = new TH2F("hNHod12","Nhod1 v.s. Nhod2",
                        16,0.,16.,16,0.,16.);
  hMonCsi   = new TH2F("hMonCsi","Mon v.s. Csi",
                        128,0.,2048.,128,0.,2048.);

  last->cd();
}

//=================================================================
//-------------------------------
void UserAnalysis::DrawCounters()
{
  TDirectory *last = gDirectory;
  gFile->cd("/");
  U4CDCRawData     *raw = (U4CDCRawData*)gJSF->FindModule("U4CDCRawData");
  U4CDCRawDataBuf  *evt = (U4CDCRawDataBuf *)raw->EventBuf();
  U4CDCBeginRunBuf *br  = raw->GetBeginRunBuf();

  Int_t              ncdc = evt->GetNADCData();
  TClonesArray   *adcdata = evt->GetADCData();

  hHod1->Reset();
  hHod2->Reset();

  Int_t    h1[16], h2[16];
  Double_t adc1[16], adc2[16], adc1pk[18], adc2pk[18];
  for (Int_t i = 0; i < 18; i++) adc1pk[i] = adc2pk[i] = 0;
  Int_t    n1 = 0;
  Int_t    n2 = 0;
  Double_t adcsum1 = 0;
  Double_t adcsum2 = 0;
  Double_t adcmon  = 0;
  Double_t adccsi  = 0;
  Double_t adctc1  = 0;
  Double_t adctc2  = 0;

  Int_t ic1st = GetDisplay()->SkipChamberData();

  for (Int_t i = ic1st; i < ncdc; i++) {
     U4CDCRawADCDatum *adcdatum = (U4CDCRawADCDatum *)adcdata->UncheckedAt(i);
     Int_t a;
     Int_t id;
     Int_t cr = 0;               // We have only 1 crate now.
     Int_t st = adcdatum->GetStationNo();
     Int_t ch = adcdatum->GetChannelNo();
     
     if (st == 15) {                  // TEMPORARY REMEDY FOR BROKEN DATA
         cerr << "Station No." << st << " required !" << endl;
         break;
     }

     br->EChannelToDChannel(cr,st,ch,id,a);
     if (id == __CHAMBERID__) continue;

     if (a > 999999) break;
 
     Double_t adc  = adcdatum->GetADC();
#if 1
cerr << "i st ch id a adc " << i << " " << st << " " << ch 
     << " " << id << " " << a << " " << adc << endl;
#endif
     if (a >=0 && a < __HODOX__+__HODOY__-1) { // hodoscopes
	if (adc > kHodPed[a] + kHodThresh) {
           hHADC[a]->Fill(adc);
           if (a < __HODOX__) {
              hHod1->Fill(a,adc);
              adc1[n1]    = adc - kHodPed[a];
              adc1pk[a+1] = adc1[n1];
              adcsum1    += adc1[n1];
              h1[n1++]    = a;
           } else {
              hHod2->Fill(a-__HODOX__,adc);
              adc2[n2]              = adc - kHodPed[a];
              adc2pk[a+1-kNhodos/2] = adc2[n2];
              adcsum2              += adc2[n2];
              h2[n2++]              = a - __HODOX__;
           }
        }
     } else if ( id != 0 && a >= kNhodos-1 ) {  // counters
        switch (a) {
           case 31: {
                adctc1 = adc;
                break;
           }
           case 33: {
                adctc2 = adc;  
                break;
           }
           case 34: {
                adcmon = adc;
                break;
           } 
           case 35: {
                adccsi = adc;
                break;
           }
           default:
                break;
        }
     }
  }

  Int_t npk1 = 0;
  Int_t npk2 = 0;
  for (Int_t i = 1; i <= kNhodos/2; i++) {
     if (adc1pk[i] > adc1pk[i-1] && adc1pk[i] >= adc1pk[i+1]) npk1++;
     if (adc2pk[i] > adc2pk[i-1] && adc2pk[i] >= adc2pk[i+1]) npk2++;
  }
  for (Int_t i = 0; i < n1; i++) {
     for (Int_t j = 0; j < n2; j++) {
        hHod12->Fill(h1[i],h2[j],adc1[i]*adc2[j]);
     }
  }
  hHodADC12->Fill(adcsum1,adcsum2);
  hTC1TC2->Fill(adctc1,adctc2);
  hMonHod->Fill(adcmon,adcsum1+adcsum2);
  hHod1hs->Fill(adcmon,adcsum1);
  hNHod12->Fill(n1+0.5,n2+0.5);
  hMonCsi->Fill(adcmon,adccsi);
  if (npk1 < 2 && npk2 < 2) {
     hHod12lo->Fill(adcsum1,adcsum2);
  } else if ( npk1 > 1 && npk2 > 1) {
     hHod12hi->Fill(adcsum1,adcsum2);
  }

  if (!(GetDisplay()->IsInLoop())) {  
     TCanvas *daparent = (TCanvas *)gROOT->FindObject("DisplayArea"); 
     daparent->cd();
     
     Int_t run = gJSF->GetRunNumber();
     Char_t runno[256];
     sprintf(runno,"Run%4.4d", run);
     
     fLabel.SetLabel(runno);
     fLabel.SetFillColor(18);
     fLabel.SetTextSize(0.99);
     fLabel.Draw();
     
     TPad *da = new TPad("da", "Counter Info",0,0,1,0.95);
     da->Draw();
      
     da->cd();
     Int_t page = GetDisplay()->GetPage();
     switch (page) {
        case 1: {
           da->Divide(1,2);
           gStyle->SetOptStat(0);
           da->cd(1);
           hHod1->Draw();
           da->cd(2);
           hHod2->Draw();
           break;
        }
        case 2: {
           da->Divide(8,4);
           for (Int_t i = 0; i < kNhodos-1; i++) {
              da->cd(i+1);
              gStyle->SetOptStat(0);
              gPad->SetTopMargin(0.01);
              gPad->SetBottomMargin(0.2);
              hHADC[i]->GetXaxis()->SetTickLength(0.04);
              hHADC[i]->GetYaxis()->SetTickLength(0.05);
              hHADC[i]->GetXaxis()->SetNdivisions(5);
              hHADC[i]->GetYaxis()->SetNdivisions(5);
              hHADC[i]->GetXaxis()->SetLabelSize(0.10);
              hHADC[i]->GetYaxis()->SetLabelSize(0.10);
              hHADC[i]->Draw();
              // da->Update();
           }
           break;
        }
        case 3: {
           da->Divide(3,4);
           gStyle->SetOptStat(0);
           da->cd(1);
           hHod12->Draw();
           da->cd(2);
           DeleteHist("hHod12_px");
           hHod12->ProjectionX()->Draw();
           da->cd(3);
           DeleteHist("hHod12_py");
           hHod12->ProjectionY()->Draw();
           da->cd(4);
           hHodADC12->Draw();
           da->cd(5);
           DeleteHist("hHodADC12_px");
           hHodADC12->ProjectionX()->Draw();
           da->cd(6);
           DeleteHist("hHodADC12_py");
           hHodADC12->ProjectionY()->Draw();
           da->cd(7);
#if 1
           DeleteHist("hTC1TC2_px");
           hTC1TC2->ProjectionX()->Draw();
#else
           hTC1TC2->Draw();
#endif
           da->cd(8);
#if 1
           hHod1hs->Draw();
#else
           hMonHod->Draw();
#endif
           da->cd(9);
           hNHod12->Draw();
           da->cd(10);
           hMonCsi->Draw();
#if 1
           da->cd(11);
           hHod12lo->Draw();
           da->cd(12);
           hHod12hi->Draw();
#endif
           break;
        }
        default:
           break;
     }
     daparent->cd(); daparent->Update();
  }
  last->cd();
}

//=================================================================
//-------------------------------------
void UserAnalysis::UpdateCounterInfo()
{
  U4CDCRawData     *raw = (U4CDCRawData*)gJSF->FindModule("U4CDCRawData");
  U4CDCRawDataBuf  *evt = (U4CDCRawDataBuf *)raw->EventBuf();
  U4CDCBeginRunBuf *br  = raw->GetBeginRunBuf();
  
  Int_t              ncdc = evt->GetNADCData();
  TClonesArray   *adcdata = evt->GetADCData();

  Int_t tc1adc = 0;
  Int_t tc2adc = 0;
  Int_t hod1sumadc = 0;
  Int_t csiadc = 0;
  for (Int_t i = 0; i < ncdc; i++) {
    U4CDCRawADCDatum *adcdatum = (U4CDCRawADCDatum *)adcdata->UncheckedAt(i);
    Int_t st = adcdatum->GetStationNo();
    if (st < __CHAMBERSTLAST__) continue;
    Int_t a;
    Int_t id;
    Int_t cr = 0;               // We have only 1 crate now.
    Int_t ch = adcdatum->GetChannelNo();

    br->EChannelToDChannel(cr,st,ch,id,a);
    if (a > 999999) break;

    Int_t adc  = (Int_t)adcdatum->GetADC();
    switch (a) {
    case 31: {
      tc1adc = (Int_t)adc;
      //      cout << "ADC of TC1 = " << tc1adc << endl;
      break;
    }
    case 33: {
      tc2adc = (Int_t)adc;
      //      cout << "ADC of TC2 = " << tc2adc << endl;
      break;
    }
    case 34: {
      hod1sumadc = (Int_t)adc;
      //      cout << "ADC of HOD1 Sum = " << hod1sumadc << endl;
      break;
    }
    case 35: {
      csiadc = (Int_t)adc;
      //      cout << "ADC of CsI = " << csiadc << endl;
      break;
    }
    default:
      break;
    }
  }
  Char_t cntinfo[256];
  if (gJSF->GetRunNumber() < 858) {
    sprintf(cntinfo,"Counter ADC : TC1 = %4.4d,  TC2 = %4.4d,  CsI = %4.4d",
            tc1adc, tc2adc, csiadc);
  } else {
    sprintf(cntinfo,"Counter ADC : TC1 = %4.4d,  HOD1Sum = %4.4d,  TC2 = %4.4d,  CsI = %4.4d",
            tc1adc, hod1sumadc, tc2adc, csiadc);
  }

  TGString *str = new TGString(cntinfo);
  GetDisplay()->UpdateCounterInfo(str);
}

//______________________________________________________________________________
// ----------
//  Steering
// ----------
//

void UserAnalysis::Initialize()
{
  // cerr << "User's Initialize() is called" << endl;
  InitRawFADC();
  InitCounters();
}

void UserAnalysis::DrawHist()
{
  Int_t page = GetDisplay()->GetPage();
 switch (page) {
     case 5:
     case 4:
     case 3:
     case 2:
     case 1:
     	DrawCounters();
        break;
     case 0:
  	DrawRawFADC();
  	break;
     default:
        break;
  }
}
