#include "BabyDisplay.h"

const Int_t kNwires   = 10;
const Int_t kNbuttons = 9;

      Int_t BabyDisplay::kNpages   = 6;

ClassImp(BabyDisplay)

//---------------------------------------------------------------------------
BabyDisplay::BabyDisplay()
	: TGMainFrame(gClient->GetRoot(), 900, 800, kHorizontalFrame), 
	  fCanvas(0), fCont(0), fBottomButtons(0),
	  fStatusBar(0), fStatus(0),
	  fMenuBar(0), fMenus(0),
          fCounterInfo(0),
	  fPage(0),
	  fIsInitialized(kFALSE), fInLoop(kFALSE)
{
   fButtons = new TGTextButton* [kNbuttons];
   fButtons[0] = 0;
   fButtons[1] = 0;
   fButtons[2] = 0;
   fButtons[3] = 0;
   fButtons[4] = 0;
   fButtons[5] = 0;
   fButtons[6] = 0;
   fButtons[7] = 0;
   fButtons[8] = 0;
}

//---------------------------------------------------------------------------
BabyDisplay::~BabyDisplay()
{
   if (fCont)   delete fCont;
   if (fCanvas) delete fCanvas;
}

//---------------------------------------------------------------------------
void BabyDisplay::Initialize()
{
    SetWindowName("Baby Chamber Event Display");

    TGLayoutHints *lhC 
    	= new TGLayoutHints(kLHintsCenterX,2,2,2,2);
    TGLayoutHints *lhTL
    	= new TGLayoutHints(kLHintsTop|kLHintsLeft|kLHintsExpandX,1,1,1,1);
    TGLayoutHints *lhTLnoexpand
    	= new TGLayoutHints(kLHintsTop|kLHintsLeft,1,1,1,1);
    TGLayoutHints *lhTRnoexpand
    	= new TGLayoutHints(kLHintsTop|kLHintsRight,1,1,1,1);
    TGLayoutHints *lhItem
    	= new TGLayoutHints(kLHintsTop | kLHintsLeft, 0, 1, 0, 0);

    fCont       = new TGCompositeFrame(this,900,700,kVerticalFrame);

// ------------------
//  Status Bar
// ------------------
    fStatusBar  = new TGCompositeFrame(fCont,100,5,kHorizontalFrame);

// ------------------
//  Status Bar Items
// ------------------
    fStatus     = new TGLabel(fStatusBar,"Initializing");
    Int_t nevt  = gJSF->GetEventNumber() + 1;
    if (nevt == 1 && gJSF->GetInput()) {
       if (gJSF->GetEvent(nevt)) {
          UpdateStatus();
    	  gJSF->Process();
       } else {	
          cerr << "Failed to get event #" << nevt << endl;
       }
    }
    fStatusBar->AddFrame(fStatus,lhTLnoexpand);

    fMenuBar    = new TGMenuBar(fStatusBar, 1, 1, kHorizontalFrame);
    fMenus      = new TGPopupMenu(fClient->GetRoot());
    fMenus->Associate(this);
    fMenus->AddEntry("New Canvas", 9001);		// Canvas
    fMenus->AddEntry("Open Browser", 9002);		// Browser
    fMenuBar->AddPopup("Tools", fMenus, lhItem);    
    fStatusBar->AddFrame(fMenuBar,lhTRnoexpand);

    fCounterInfo = new TGLabel(fStatusBar,"Initializing");
    if (nevt == 1 && gJSF->GetInput()) {
      if (gJSF->GetEvent(nevt)) {
	UpdateCounterInfo();
	gJSF->Process();
      } else {
	cerr << "Failed to get event #" << nevt << endl;
      }
    }
    fStatusBar->AddFrame(fCounterInfo,lhC);


    fCont->AddFrame(fStatusBar,lhTL);

// ------------------
//  Display Area
// ------------------
    fCanvas = new TRootEmbeddedCanvas("DisplayArea",fCont,800,650);
    fCont->AddFrame(fCanvas,lhC);

// ------------------
//  Bottom Buttons
// ------------------
    fBottomButtons = new TGCompositeFrame(fCont,10,10,kHorizontalFrame);
    fButtons[1] = new TGTextButton(fBottomButtons, "Prev_Page",8002);
    fButtons[0] = new TGTextButton(fBottomButtons, "Page 1",8003);
    fButtons[2] = new TGTextButton(fBottomButtons, "Reset", 8004);
    fButtons[3] = new TGTextButton(fBottomButtons, "Next",  8005);
    fButtons[4] = new TGTextButton(fBottomButtons, "Prev",  8006);
    fButtons[5] = new TGTextButton(fBottomButtons, "Go",    8007);
    fButtons[6] = new TGTextButton(fBottomButtons, "Print", 8008);
    fButtons[7] = new TGTextButton(fBottomButtons, "Quit",  8009);
    fButtons[8] = new TGTextButton(fBottomButtons, "PrintCounter",  8010);

    for (Int_t i=0; i<kNbuttons; i++) {
       fButtons[i]->Associate(this);
       fBottomButtons->AddFrame(fButtons[i],lhTL);
    }
    fCont->AddFrame(fBottomButtons,lhTL);

    AddFrame(fCont,lhC);
    fCont->Layout();
    fCanvasDirectory=gDirectory;

    MapSubwindows();
    Resize(GetDefaultSize());
    Layout();
    MapWindow();
    
    gROOT->LoadMacro("UserMacro.C");
    gROOT->ProcessLine("useranl->Initialize();");
    fIsInitialized = kTRUE;
    Display();
}

//---------------------------------------------------------------------------
void BabyDisplay::Display()
{
   if (!fIsInitialized) {
      Initialize();
   } else {
      fCanvas->GetCanvas()->cd();
      gPad->Clear();
      fCanvas->Clear();
   } 
   TDirectory *olddir = gDirectory;
   fCanvasDirectory->cd();

   fCanvas->GetCanvas()->cd();
   gROOT->ProcessLine("useranl->DrawHist();");

   if (fCanvasDirectory != olddir) olddir->cd();
}

//---------------------------------------------------------------------------
void BabyDisplay::ProcessButton(Long_t parm1)
{
   switch(parm1) {
      case 8009: { // Quit
         gJSF->Terminate();
         gJSF->GetOutput()->Write();
         CloseWindow();   // this also terminates theApp
         gROOT->ProcessLine(".q;");
         break;
      }
      case 8008: { // SaveAs
         Char_t fname[256];
         sprintf(fname,"run%4.4d.evt%5.5d.eps",
         		gJSF->GetRunNumber(), gJSF->GetEventNumber());
         gPad->SaveAs(fname);
         break;
      }
      case 8007: { // Go
         // cerr << "Go button is pressed" << endl;
         Int_t nevt = gJSF->GetEventNumber() + 1;
         Int_t nnext;
         cout << "*********************************************************" << endl;
         cout << "* How many events do you want? Type 0 to see all events *" << endl;
         cout << "* ----------------------------------------------------- *" << endl;
         cin  >> nnext;
         cerr << nnext;
         cout << "* " << nnext << " is accepted.                          *" << endl;
         cout << "*********************************************************" << endl;
#if 1
	 cerr << "Event " << nnext << endl;
#endif

	 if (nnext==0) {
           while (gJSF->GetEvent(nevt++)) {
              fInLoop = kTRUE;
              cerr << "Event No " << nevt << endl;
	      gJSF->Process();
              Display();
           }
         } else {
           for (Int_t i = nevt; i< nevt+nnext; i++) {
              if(gJSF->GetEvent(i)){
                fInLoop = kTRUE;
                cerr << "Event No " << i << endl;
	        gJSF->Process();
                Display();
              }  
           }
         }
         
         fInLoop = kFALSE;
         UpdateStatus();
	 UpdateCounterInfo();
         Display();
         break;
      }
      case 8006: { // Prev
         // cerr << "Prev button is pressed" << endl;
         Int_t nevt = gJSF->GetEventNumber() - 1;
#if 1
	 cerr << "Event " << nevt << endl;
#endif
	 if (nevt+1 > 0) gJSF->Clear();
         if (gJSF->GetInput()) {
            if (gJSF->GetEvent(nevt)) {
	      UpdateStatus();
	      UpdateCounterInfo();
	    }
            else	    cerr << "Failed to get event #" << nevt << endl;
         }
         gJSF->Process();
         Display();
         break;
      }
      case 8005: { // Next
         // cerr << "Next button is pressed" << endl;
         Int_t nevt = gJSF->GetEventNumber() + 1;
#if 1
	 cerr << "Event " << nevt << endl;
#endif
	 if (nevt-1 > 0) gJSF->Clear();
         if (gJSF->GetInput()) {
            if (gJSF->GetEvent(nevt)) {
	      UpdateStatus();
	      UpdateCounterInfo();
	    }
            else	    cerr << "Failed to get event #" << nevt << endl;
         }
         gJSF->Process();
         Display();
         break;
      }
      case 8004: { // Reset
         // cerr << "Reset button is pressed" << endl;
         Int_t nevt = 1;
         gJSF->SetEventNumber(nevt);
#if 1
	 cerr << "Event " << nevt << endl;
#endif
	 if (nevt+1 > 0) gJSF->Clear();
         if (gJSF->GetInput()) {
            if (gJSF->GetEvent(nevt)) {
	      UpdateStatus();
	      UpdateCounterInfo();
	    }
            else	    cerr << "Failed to get event #" << nevt << endl;
         }
         gJSF->Process();
         Display();
         break;
      }
      case 8003: { // Page
         fPage++;
         fPage = fPage % kNpages;
         Char_t label[256];
         sprintf(label, "Page %i", fPage+1);
         fButtons[0]->SetText(new TGHotString(label));
         // cerr << "Switch to Page " << fPage << endl;
         Display();
         break;
      }
      case 8002: { // Prev_Page
         fPage--;
         if(fPage < 0) fPage += kNpages;
         Char_t label[256];
         sprintf(label, "Page %i", fPage+1);
         fButtons[0]->SetText(new TGHotString(label));
         // cerr << "Switch to Page " << fPage << endl;
         Display();
         break;
      }
      case 8010: { // PrintCounter
         fPage = 2;
         Display();
         Int_t nevt = gJSF->GetEventNumber() + 1;
#if 1
	 cerr << "Event " << nevt << endl;
#endif
         while (gJSF->GetEvent(nevt++)) {
            fInLoop = kTRUE;
            cerr << "Event No " << nevt << endl;
	    gJSF->Process();
            Display();
         }
         fInLoop = kFALSE;
         UpdateStatus();
	 UpdateCounterInfo();
         Display();
         Char_t fname[256];
         sprintf(fname,"run%4.4d.evt%5.5d.counter.eps",
         		gJSF->GetRunNumber(), gJSF->GetEventNumber());
         gPad->SaveAs(fname);
         gJSF->Terminate();
         gJSF->GetOutput()->Write();
         CloseWindow();   // this also terminates theApp
         gROOT->ProcessLine(".q;");
         break;
      }
      default:
         break;
   }
}

//---------------------------------------------------------------------------
Bool_t BabyDisplay::ProcessMessage(Long_t msg, Long_t parm1, Long_t)
{
   switch (GET_MSG(msg)) {
      case kC_COMMAND:
         switch (GET_SUBMSG(msg)) {
            case kCM_BUTTON:
                if (parm1 >= 8000) ProcessButton(parm1);
                break;
            case kCM_MENUSELECT:
               break;
            case kCM_MENU:
               switch (parm1) {
                  case 9001: {
                     TCanvas *c;
                     c = new TCanvas();
                     break;
                  }
                  case 9002: {
                     TBrowser *b;
                     b = new TBrowser();
                     break;
                  }
                  default:
                     break;
               }
            default:
               break;
         }
         break;
      default:
         break;
   }
   return kTRUE;
}

//---------------------------------------------------------------------------
void BabyDisplay::UpdateStatus()
{
    //Char_t status[256];
    Char_t *status = new Char_t [256];
    sprintf(status,"Run %4.4d  Event %5.5d",
         		gJSF->GetRunNumber(), gJSF->GetEventNumber());
    fStatus->SetText(new TGString(status));
}

//---------------------------------------------------------------------------
void BabyDisplay::UpdateCounterInfo()
{
    gROOT->ProcessLine("useranl->UpdateCounterInfo();");
}

//---------------------------------------------------------------------------
void BabyDisplay::UpdateCounterInfo(TGString *str)
{
  fCounterInfo->SetText(str);
}

//---------------------------------------------------------------------------
Int_t BabyDisplay::SkipChamberData()
{
   TDirectory *last = gDirectory;
   gFile->cd("/");
   U4CDCRawData     *raw = (U4CDCRawData*)gJSF->FindModule("U4CDCRawData");
   U4CDCRawDataBuf  *evt = (U4CDCRawDataBuf *)raw->EventBuf();

   Int_t          ncdc     = evt->GetNADCData();
   Int_t i;
   if (evt->GetNonFADCIndex() >= 0) return evt->GetNonFADCIndex();
   else return ncdc;
   last->cd();
   return i;
}
