#ifndef __BABYDISPLAY__
#define __BABYDISPLAY__

#include <iostream.h>

#include <TROOT.h>
#include <TRootEmbeddedCanvas.h>
#include <TGButton.h>
#include <TGLabel.h>
#include <TGMenu.h>
#include <TSystem.h>
#include <TCanvas.h>
#include <TPad.h>
#include <TDirectory.h>
#include <TFile.h>
#include <TBrowser.h>

#include "JSFSteer.h"
#include "U4CDCRawData.h"



class BabyDisplay : public TGMainFrame {
friend class UserAnalysis;
private:
  TDirectory          *fCanvasDirectory;	//!

  TRootEmbeddedCanvas *fCanvas;			//!
  TGCompositeFrame    *fCont;                   //!
  TGCompositeFrame    *fBottomButtons;          //!
  TGCompositeFrame    *fStatusBar;              //!
  TGLabel             *fStatus;                 //!
  TGTextButton       **fButtons;		//!
  TGMenuBar           *fMenuBar;                //!
  TGPopupMenu         *fMenus;                  //!
  TGLabel             *fCounterInfo;            //!
  Int_t                fPage;			//!
  
  Bool_t	       fIsInitialized;		//!
  Bool_t               fInLoop;                 //!
  static Int_t         kNpages;                 //!
  
private:
  void   UpdateStatus();
  void   UpdateCounterInfo();

  static void   SetNpages(Int_t n) { kNpages = n; }

public:
  BabyDisplay();
  virtual ~BabyDisplay();

  void   Initialize();
  void   ProcessButton(Long_t parm1);
  Bool_t ProcessMessage(Long_t msg, Long_t parm1, Long_t);
  void   Display();
  void   UpdateCounterInfo(TGString *str);
  
  Int_t  GetPage () const { return fPage;   }
  Bool_t IsInLoop() const { return fInLoop; }
 
  Int_t  SkipChamberData();

  ClassDef(BabyDisplay, 1)   // A class to display event
};

#endif
