
#include "U4CDCKalDetector.h"
#if __GNUC__ >= 5 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3)
#include <cstdlib>
#endif

U4CDCKalDetector * U4CDCKalDetector::fgInstance = 0;
Bool_t             U4CDCKalDetector::fgIsAxialOnly = kFALSE;

ClassImp(U4CDCKalDetector)

// ------------------------------------------------------------------
// A very crude model of JLC Detector
// ------------------------------------------------------------------
//

U4CDCKalDetector::U4CDCKalDetector() 
                : U4VKalDetector()
{
   if (fgInstance) {
      std::cerr << ">>>>>>>>> U4CDCKalDetector::U4CDCKalDetector() >>>>>>" << std::endl
                << "  U4CDCKalDetector is a singleton!!! " << std::endl;
                abort();
   } else {
      fgInstance = this;
   }

   SetBfield(30.);   // B [kG]

   fTwistedAngle[0]   =  0.   ; // axial
   fTwistedAngle[1]   =  0.512; // stereo
   fTwistedAngle[2]   = -0.475; // stereo
   fTwistedAngle[3]   =  0.   ; // axial
   fTwistedAngle[4]   =  0.415; // stereo
   fTwistedAngle[5]   = -0.394; // stereo
   fTwistedAngle[6]   =  0.   ; // axial
   fTwistedAngle[7]   =  0.358; // stereo
   fTwistedAngle[8]   = -0.344; // stereo
   fTwistedAngle[9]   =  0.   ; // axial

   if (IsAxialOnly()) {
      for (Int_t i =0; i<10; i++) {
         fTwistedAngle[i] = 0.; // axial only
      }
   }

   fLayerPhiOffset[0] =  0.   ; // axial
   fLayerPhiOffset[1] =  0.032; // stereo
   fLayerPhiOffset[2] = -0.032; // stereo
   fLayerPhiOffset[3] =  0.   ; // axial
   fLayerPhiOffset[4] =  0.032; // stereo
   fLayerPhiOffset[5] = -0.032; // stereo
   fLayerPhiOffset[6] =  0.   ; // axial
   fLayerPhiOffset[7] =  0.016; // stereo
   fLayerPhiOffset[8] = -0.016; // stereo
   fLayerPhiOffset[9] =  0.   ; // axial

   fSuperLayerPhiOffset[0] =  0    ;
   fSuperLayerPhiOffset[1] =  0    ;
   fSuperLayerPhiOffset[2] = -0.016;
   fSuperLayerPhiOffset[3] =  0.016;

   fNcellsParLayer[0]      = 42;
   fNcellsParLayer[1]      = 63;
   fNcellsParLayer[2]      = 84;
   fNcellsParLayer[3]      = 105;

   fSuperLayerInnerGap     =   4;
   fSuperLayerOuterGap     =   4;
   fAxialToStereoGap       =   5;
   fStereoToStereoGap      =   3;
   fCDCInnerR              =  44;
   fCDCOuterR              = 157;
   fCDCHalfZ               = 158;
   fThick                  = 1.;
   fDummyThick             = 1.;
   fSupportTubHalfThick    = 0.25;
   fNsuperLayers           = 4;
   fNlayers                = 10;
   fNwires                 = 5;
   fCellThick              = fThick * fNwires + 2 * fDummyThick;

   fStereoToAxialGap       = ((fCDCOuterR - 2 * fSupportTubHalfThick) 
                           - (fCDCInnerR + 2 * fSupportTubHalfThick)
                           - fSuperLayerInnerGap - fSuperLayerOuterGap
                           - fNlayers * fCellThick
                           - (fNsuperLayers - 1) 
                           * (fAxialToStereoGap + fStereoToStereoGap))
                           / (fNsuperLayers - 1);

   fLayerR[0] = fCDCInnerR + 2 * fSupportTubHalfThick + fSuperLayerInnerGap;

   // create measurement layers of central tracker

   Int_t layer;
   Int_t slayer;
   for (layer=0; layer<fNlayers; layer++) {

      if (layer) {
         switch (layer%3) {
            case 0:
               fLayerR[layer] = fLayerR[layer-1] + fCellThick + fStereoToAxialGap;
               break;
            case 1:
               fLayerR[layer] = fLayerR[layer-1] + fCellThick + fAxialToStereoGap;
               break;
            case 2:
            default:
               fLayerR[layer] = fLayerR[layer-1] + fCellThick + fStereoToStereoGap;
         }
      }

      if (!layer) slayer = 0;
      else        slayer = ((layer - 1) / 3) + 1;

      for (Int_t wire=0; wire<fNwires; wire++) {

         Double_t r = fLayerR[layer] + fThick * (wire + 0.5) + fDummyThick;
         Double_t tana = r/fCDCHalfZ * TMath::Sin(fTwistedAngle[layer]/2);
         Double_t r0 = TMath::Sqrt(r * r - fCDCHalfZ * fCDCHalfZ * tana * tana);
         TVector3 we(r, 0., -fCDCHalfZ);
         Double_t rot = (-fTwistedAngle[layer]/2 + fLayerPhiOffset[layer] + fSuperLayerPhiOffset[slayer]);
         we.RotateZ(rot);
         TVector3 wt(r, 0.,  fCDCHalfZ);
         rot = (fTwistedAngle[layer]/2 + fLayerPhiOffset[layer] + fSuperLayerPhiOffset[slayer]);
         wt.RotateZ(rot);
         TVector3 wd = wt - we;
         Double_t celw = 2.*r*TMath::Pi()/fNcellsParLayer[slayer];

         Add(new U4CDCMeasLayer(r0, fCDCHalfZ, we, wd.Unit(), celw));
      }
   } 
   SetOwner();
}

Double_t U4CDCKalDetector::CalcRadLength(const TVMeasLayer &from,
                                      const TVMeasLayer &to) const
{
   Double_t xl = 1.8182e+4; // [cm] for CO2/iso-C4H10 (90:10)
   return xl;
}

U4CDCKalDetector * U4CDCKalDetector::GetInstance()
{
   if (!fgInstance) {
      fgInstance = new U4CDCKalDetector;
   }
   return fgInstance;
}

Double_t U4CDCKalDetector::CalcSigmaMS0(const TVMeasLayer &vfrom,
                                        const TVMeasLayer &vto,
                                              Double_t     pathlen) const
{
   static const Double_t kMS1  = 0.0136;
   static const Double_t kMS2  = 0.038;

   const U4CDCMeasLayer &from = *dynamic_cast<const U4CDCMeasLayer *> (&vfrom);
   const U4CDCMeasLayer &to   = *dynamic_cast<const U4CDCMeasLayer *> (&vto);

   Double_t xl    = pathlen / CalcRadLength(from,to);
   // ------------------------------------------------------------------
            xl    = TMath::Max(xl,1.e-4); // very crude treatment that
                                          // should be improved!
   // ------------------------------------------------------------------

   Double_t sgms0 = kMS1 * TMath::Sqrt(xl) * (1 + kMS2*TMath::Log(xl));

   if (IsMSOn()) return sgms0;
   else          return 0.;
}

Double_t U4CDCKalDetector::GetAvRadius() const
{
   return dynamic_cast<U4CDCMeasLayer *>(this->At(0))->GetR0();
}

