#ifndef __U4CDCKalHIT__
#define __U4CDCKalHIT__

#include <iostream>
#include <iomanip>
#include "KalTrackDim.h"
#include "TVTrackHit.h"
#include "U4CDCMeasLayer.h"

using namespace std;

class U4CDCKalHit : public TVTrackHit {
public:
   U4CDCKalHit(Int_t m = kMdim);
   U4CDCKalHit(const U4CDCMeasLayer &ms, Double_t *x, Double_t *dx, Int_t lr, Int_t cn,
         Double_t v, Double_t b = 30., Int_t m = kMdim);
   U4CDCKalHit(const U4CDCMeasLayer &ms, Double_t *x, Double_t *dx, Int_t lr, Int_t cn,
         Double_t v, const TVector3 &xx, Double_t b = 30., Int_t m = kMdim);
   U4CDCKalHit(const U4CDCKalHit &hit);

   ~U4CDCKalHit();

   inline Int_t    GetLR()        const { return fLR;                         }
   inline Int_t    GetCellNo()    const { return fCellNo;                     }
   inline TVector3 GetWireEnd()   const
   { 
      return GetCDCMeasLayer().GetWireEnd(fCellNo);
   } 
   inline TVector3 GetWireDir()   const
   {
      return GetCDCMeasLayer().GetWireDir(fCellNo);
   } 
   inline Double_t GetVdrift()    const { return fVdrift;                     }

   inline void     SetVdrift (Double_t v)         { fVdrift   = v;  }

   virtual TKalMatrix XvToMv (const TVector3 &xv, Double_t t0)  const;

   virtual void       DebugPrint(Option_t *opt = "")        const;

private:
   inline const U4CDCMeasLayer & GetCDCMeasLayer() const
   {
      return *(const U4CDCMeasLayer *)&TVTrackHit::GetMeasLayer();
   }

private:
   Int_t        fLR;           // (left,right) = (-1,+1)
   Int_t        fCellNo;       // cell No
   Double_t     fVdrift;       // drift velocity

   ClassDef(U4CDCKalHit,1)      // Sample hit class
};

#endif
