//*************************************************************************
//* ---------------------
//* U4CDCHitCell Class
//* ---------------------
//* (Description)
//*	Class to store "U4CDCHitWire"s for a single cell.
//* (Requires)
//*	TObjArray
//* (Provides)
//*	U4CDCHitCell		: Array containing "U4CDCHitWire"s
//* (Update Record)
//*	1999/12/04  K.Fujii	Original version.
//*************************************************************************

#include <iostream>
#include <iomanip>
#include "U4CDCHitCell.h"

ClassImp(U4CDCHitCell)

U4CDCHitCell::U4CDCHitCell() : TObjArray(5),
	fLayerNumber(0), fCellNumber(0),
	fLayerPtr(0) 
{
}

U4CDCHitCell::~U4CDCHitCell()
{
	Delete();
}

Bool_t U4CDCHitCell::IsSortable() const { return kTRUE; }

Int_t U4CDCHitCell::Compare(const TObject *obj) const
{
	Int_t cell  = fCellNumber;
	Int_t cello = ((U4CDCHitCell *)obj)->fCellNumber;
   	if (cell < cello) return -1;
   	else if (cell > cello) return 1;
   	else return 0;
}

Int_t U4CDCHitCell::Append(U4CDCHitWire *wire)
{	
	Int_t layer = wire->GetLayerNumber();
	Int_t cell  = wire->GetCellNumber();
	
	// Return value = -1:	new wire.
	//		=  0:	successfully appended.
	//		=  1:	new cell.
	//		=  2:	new layer.
	
	if (GetEntries() == 0) {
		fLayerNumber   = layer;
		fCellNumber    = cell;
		wire->fCellPtr = this;
                Add(wire);
                return 0;
	} else {
		if (layer == fLayerNumber) {
			if (cell == fCellNumber) {
				wire->fCellPtr = this;
                        	Add(wire);
                        	return 0;
                        } else {
                        	return 1;
                        }
                } else {
                        return 2;
                }
        }
}

void U4CDCHitCell::DebugPrint(const Char_t *opt)
{
	std::cerr 	<< "(layer,cell) = " 
		<< "(" << fLayerNumber << "," << fCellNumber << ")" << std::endl;
	TIter next(this);
	U4CDCHitWire *wire;
	Int_t n = 0;
	while ((wire = (U4CDCHitWire *)next())) {
		std::cerr << std::setw(3) << ++n << " ";
		wire->DebugPrint(opt);
	}
}
