#ifndef __U4CDCHITMAKERBUF__
#define __U4CDCHITMAKERBUF__

//*************************************************************************
//* ---------------------
//* U4CDCHitMakerBuf Class
//* ---------------------
//* (Description)
//*	This set of classes makes FADC clusters.
//* (Requires)
//*	JSFEventBuf
//* (Provides)
//*	U4CDCHitMakerBuf		: buffering class
//* (Update Record)
//*	1999/12/04  K.Fujii	Original version.
//*************************************************************************

#include "U4VHitMakerBuf.h"
#include "U4CDCHitMaker.h"
#include "U4CDCHitLayer.h"
#include "U4CDCClusterBuf.h"


class U4CDCHitMakerBuf : public U4VHitMakerBuf {

friend class U4CDCHitMaker;

public:

  U4CDCHitMakerBuf(U4CDCTimeToXRelation *calib = 0,
  	           const char *name="U4CDCHitBuf",
	           const char *title="CDC Hit data", 
	           U4CDCHitMaker *module=0, Int_t bufsize = 2000);
  U4CDCHitMakerBuf(U4CDCHitMaker *module,
  	           U4CDCTimeToXRelation *calib,
	           const char *name="U4CDCHitBuf",
	           const char *title="CDC Hit data",
	           Int_t bufsize = 2000);
		    
  virtual ~U4CDCHitMakerBuf();
  
  virtual void 	DebugPrint(const Char_t *opt = "");
  
  void          SetClusterData(U4CDCClusterBuf *clbuf);
  void		SetCalibrationData(U4CDCTimeToXRelation *calib);
  inline void	SetWidthCut(Double_t x) { xWidth = x; }
  inline void	SetPeakCut (Double_t x) { xPeak  = x; }
  inline void	SetSumCut  (Double_t x) { xSum   = x; }
  
protected:
  virtual void	MakeHits() ;	// Make CDC Hits
  
private:
  Double_t	        xWidth;		//  width cut
  Double_t	        xPeak;		//  peak cut
  Double_t	        xSum;		//  sum cut
  U4CDCClusterBuf      *fClusterData;   //  Array of clusters
  U4CDCTimeToXRelation *fCalibPtr;	//! pointer to Hit calibration data

  ClassDef(U4CDCHitMakerBuf, 1) // Cluster data buffer
};



#endif
