//*************************************************************************
//* --------------------
//* U4CDCTrackMakerBuf
//* --------------------
//* (Description)
//*	This set of classes makes pad Tracks.
//* (Requires)
//* (Provides)
//* (Update Record)
//*	2002/11/15  R.Kuboshima	   Original version.
//*************************************************************************

#include <iostream>
#include <iomanip>
#include "U4CDCTrackMakerBuf.h"
#include "U4CDCTrack.h"
#include "U4CDCVectorHit.h"


ClassImp(U4CDCTrackMakerBuf)

//=====================================================================
//* constructor -------------------------------------------------------
U4CDCTrackMakerBuf::U4CDCTrackMakerBuf(const Char_t          *name,
                                       const Char_t          *title, 
			                     U4CDCTrackMaker *module,
                                             Int_t            bufsize) 
		   : U4VTrackMakerBuf(name, title, module, bufsize),
	             xNsigmas(4.),
                     xChi2(45.),
                     xDXpivot(0.05),
	             xKCNsigmas(2.),
                     xSlope(9999999.),
                     fBmin(-5.5),
                     fBmax(5.5),
                     fNbins(100),
                     fNvhitCut(2),
                     fJunkVHit(20)
{
}

U4CDCTrackMakerBuf::U4CDCTrackMakerBuf(      U4CDCTrackMaker *module,
	                               const Char_t          *name, 
			               const Char_t          *title,
                                             Int_t            bufsize)
		   : U4VTrackMakerBuf(module, name, title, bufsize),
                     xNsigmas(4.),
                     xChi2(45.),
                     xDXpivot(0.05),
		     xKCNsigmas(2.),
                     xSlope(9999999.),
                     fBmin(-5.5),
                     fBmax(5.5),
                     fNbins(100),
                     fNvhitCut(2),
                     fJunkVHit(20)
{
}

//=====================================================================
//* destructor --------------------------------------------------------

U4CDCTrackMakerBuf::~U4CDCTrackMakerBuf()
{  
}

//=====================================================================
//* MakeTracks --------------------------------------------------------

void U4CDCTrackMakerBuf::MakeTracks()
{
   if (!IsInitialized()) {
      std::cerr << "ERROR  U4CDCTrackMakerBuf::MakeTracks"   << std::endl
                << "...... cluster data not yet initialized." << std::endl;
      return;
   } else if (fDone) {
      return;
   }

   fDone = kTRUE;

   TObjArray *inputbufp = GetHitMakerBufPtr()->GetOutputBufPtr();

   TObjArray truevhittmp(1000);    
        
   TIter nextlayer(inputbufp);
   U4CDCHitLayer *lyrp;
   while ((lyrp = (U4CDCHitLayer *)nextlayer())) {
      TIter nextcell(lyrp);
      U4CDCHitCell *celp;
      while ((celp = (U4CDCHitCell *)nextcell())) {
         MakeVectorHits(celp, &truevhittmp);
      }
   }
        
   if (truevhittmp.GetEntries() == 0) {
      std::cerr << "***U4CDCTrackMakerBuf::MakeTracks: no truevhit!***" 
                << std::endl;
      return;
   }

   TIter nextvhit(&truevhittmp);
   U4CDCVectorHit *vhitp;
#if 1
   std::cerr << "U4CDCTrackMakerBuf::MakeTracks: truevhittmp has "
             << truevhittmp.GetEntries() << " vhitp."
             << std::endl;

   while ((vhitp = (U4CDCVectorHit *)nextvhit())) {
      std::cerr << "   vhitp = " << vhitp 
                << ",  layerno = " << vhitp->GetLayerNumber()
                << ",  cellid = " << vhitp->GetCellID()
                << std::endl;
   }
   nextvhit.Reset();
#endif
#if 0
   // --------------
   //  Fill b-array
   // --------------
   TObjArray barray(fNbins);
   Double_t db = (fBmax - fBmin) / fNbins;
   while ((vhitp = (U4CDCVectorHit *)nextvhit())) {
      Double_t b = vhitp->GetIntercept();
      std::cerr << "b = " << b << std::endl; 
      if (b < -5.5 || b > 5.5) {
         std::cerr << "Kill vhitp = " << vhitp << std::endl; 
         delete vhitp;
	 continue;
      } else {
         Int_t    i = (Int_t)((b - fBmin) / db);
         if (!barray[i]) barray[i] = new U4CDCTrack();
         std::cerr << "Add vhitp in barray[" << i << "]" << " = " << vhitp
                   << std::endl; 
         ((U4CDCTrack *)barray[i])->Add(vhitp);
      }
   }

   // --------------
   //  Merge b-array
   // --------------
   for (Int_t i = 1; i < fNbins; i++) {
      if (barray[i]   && ((U4CDCTrack *)barray[i])->GetEntries() && 
	  barray[i-1] && ((U4CDCTrack *)barray[i-1])->GetEntries()) {
         TIter next((U4CDCTrack *)barray[i-1]);
	 U4CDCVectorHit *vhitp;
         while ((vhitp = (U4CDCVectorHit *)next())) {
               std::cerr << "Marge vhitp in barray[" 
		         << i << "]" << " = " << vhitp
                         << std::endl; 
               ((U4CDCTrack *)barray[i])->Add(new U4CDCVectorHit(*vhitp));
         }
         delete barray[i-1];
	 barray[i-1] = 0;
      }
   }

   // --------------
   //  Find tracks
   // --------------
   Int_t trackno = 0;
   for (Int_t i = 0; i < fNbins; i++) {
      if (!barray[i]) continue;
      U4CDCTrack *tkp = (U4CDCTrack *)barray[i];
      tkp->SetOwner();
      Int_t nvhits = tkp->GetEntries();
      if (nvhits >= fNvhitCut) {
      //------------------------
      // Fit and collect vhitp.
      //------------------------
	 std::cerr << vhitp << " has more 2 vhits (^^)" << std::endl;
         U4CDCTrack *bst = 0;
         for (Int_t iv = 0; iv < nvhits-1; iv++) {
	    U4CDCVectorHit *vhitp1 = (U4CDCVectorHit *)tkp->At(iv); 
            Int_t layerno1 = vhitp1->GetLayerNumber();
	    std::cerr << vhitp1 << "'s layerno = " << layerno1 << std::endl;
	    Double_t lstchi2 = 1000000000.;
	    for (Int_t jv = iv+1; jv < nvhits; jv++) {
	       std::cerr << "jv = " << jv << std::endl;
	       U4CDCVectorHit *vhitp2 = (U4CDCVectorHit *)tkp->At(jv); 
	       std::cerr << "vhitp2 = " << vhitp2 << std::endl;
               Int_t layerno2 = vhitp2->GetLayerNumber();
	       std::cerr << vhitp2 << "'s layerno = " << layerno2 << std::endl;
               if (layerno1 == layerno2) continue; 
               U4CDCTrack *tmptkp = new U4CDCTrack();
               tmptkp->Add(new U4CDCVectorHit(*vhitp1));
               tmptkp->Add(new U4CDCVectorHit(*vhitp2));
	       tmptkp->SetOwner();
               tmptkp->Update();
               Double_t chi2 = tmptkp->GetChi2();
	       std::cerr << "chi2 =  " << chi2 
	                 << ",  lstchi2 =  " << lstchi2 << std::endl;
               if (chi2 < lstchi2) {
                  bst = tmptkp;
	          vhitp1->SetTrackPtr(tmptkp); 
	          vhitp2->SetTrackPtr(tmptkp); 
                  lstchi2 = chi2;
               } else {
	          delete tmptkp;
	       } 
            }
	 }

	 std::cerr << "bst = " << bst << std::endl;
      //------------------------
	 if (bst) bst->SetTrackNumber(trackno++); 
         fOutputBuf.Add(bst);
	 delete barray[i];
      } else {
         std::cerr << "Add barray[" << i << "] in fJunkVhit!!!" 
                   << std::endl; 
         fJunkVHit.Add(barray[i]);
      }
   }
#endif
   std::cerr << "****End U4CDCTrackMakerBuf::MakeTracks ****" 
             << std::endl; 
}

//=====================================================================
//* MakeVectorHits  ---------------------------------------------------

void U4CDCTrackMakerBuf::MakeVectorHits(U4CDCHitCell *celp, TObjArray *truevhittmpp)
{
   ////// Very Temporary Treatment //////
   Double_t cellwidth  = 5.0;
   Double_t cellmargin = 0.1;
   ////// Very Temporary Treatment //////  

   Int_t nwires = celp->GetEntries();
   if (nwires < 4)    return;
        
   celp->Sort();          //sort wires

   for (Int_t lrsel = 0; lrsel < 2 ; lrsel++) { //center closs(1) or not(0)
      Int_t pivotcomb;
      for (pivotcomb = 0; pivotcomb < (nwires < 5 ? 1 : 3); pivotcomb++) {
         Int_t iw1st;
         Int_t iwlst;
         switch (pivotcomb) {
            case 1:
               iw1st = 0;
               iwlst = nwires - 2;
               break;
            case 2:
               iw1st = 1;
               iwlst = nwires - 1;
               break;
            case 0:
            default:
               iw1st = 0;
               iwlst = nwires - 1;
               break;
         }   

         U4CDCHitWire *wire1st = (U4CDCHitWire *)celp->At(iw1st);
         U4CDCHitWire *wirelst = (U4CDCHitWire *)celp->At(iwlst);
   
         TIter nextin1st(wire1st);
         TIter nextinlst(wirelst);

         U4CDCHit *ht1st;
         U4CDCHit *htmdl;
         U4CDCHit *htlst;

	 nextin1st.Reset();
         while ((ht1st = (U4CDCHit *)nextin1st())) {  //hits in 1st wire
            if (ht1st->IsLocked()) continue;           //skip if locked
            Double_t x1  = ht1st->Xdrift();
            if (x1 < -cellmargin || x1 > cellwidth+cellmargin) continue;
            Double_t dx1 = ht1st->DXdrift();
            if (dx1 > xDXpivot) continue;              //skip if bad quality
            nextinlst.Reset();
            while ((htlst = (U4CDCHit *)nextinlst())) { //hits in lst wire
               if (htlst->IsLocked()) continue;          //skip if locked 
               Double_t x2  = htlst->Xdrift();
               if (x2 < -cellmargin || x2 > cellwidth+cellmargin) continue;
               Double_t dx2 = htlst->DXdrift();
               if (dx2 > xDXpivot) continue;             //skip if bad quality

               U4CDCVectorHit *trialvhitp = new U4CDCVectorHit(); //make trial track
               trialvhitp->Add(new U4CDCTrackHit(ht1st,-1)); //add 1st hit
               trialvhitp->Add(new U4CDCTrackHit(htlst,2*lrsel-1)); //add lst hit
               if (lrsel == 1) trialvhitp->Update();
               Int_t i;
               for (i = 0; i < nwires; i++) {    // loop over middle wires
                  if (i == iw1st || i == iwlst) continue;  //skip pivots
                  Double_t diffmn = 999999.;
                  Int_t lrexpect;
                  U4CDCHitWire *wiremdl = (U4CDCHitWire *)celp->At(i);
                  if (lrsel == 0) {
                     lrexpect = -1;
                  } else {
                     lrexpect = (trialvhitp->Xat(wiremdl->Y()) < 0 ? -1 : 1);
                  }
                  TIter nextinmdl(wiremdl);
                  U4CDCHit *bst = 0;
                  while ((htmdl = (U4CDCHit *)nextinmdl())) {//hits in mdl wire
                     if (htmdl->IsLocked()) continue;   //skip if locked
                     Double_t x3 = htmdl->Xdrift();
                     if (x3 < -cellmargin || x3 > cellwidth+cellmargin) continue;
                     Double_t dx3 = htmdl->DXdrift();
                     if (dx3 > xDXpivot) continue;
                     U4CDCTrackHit *hitp = new U4CDCTrackHit(htmdl,lrexpect);
                     hitp->Lock();           // exclude this hit from fit
                     trialvhitp->Add(hitp);  // add this temporarilly to track 
                     trialvhitp->Update();   // and update residual
                     Double_t diff   = hitp->GetResidual();
                     Double_t diffct = xNsigmas*(dx1+dx2+dx2);
                     trialvhitp->Remove(hitp);  // remove this hit from track
                     delete hitp;               // delete this temp. hit
                     if (TMath::Abs(diff) > diffct) continue;  //resid. cut
                     if (TMath::Abs(diff) < diffmn) {     //select the best
                        diffmn = TMath::Abs(diff);
                        bst    = htmdl;
                     }
                  }
                  if (bst) {                       // add the best to track
                     trialvhitp->Add(new U4CDCTrackHit(bst,lrexpect));
                  }
               }
               if (trialvhitp->GetEntries() < 4 
                   || trialvhitp->GetChi2() > xChi2
                   || TMath::Abs(trialvhitp->GetSlope()) > xSlope) {
                  delete trialvhitp;               // track rejected
               } else {
                  truevhittmpp->Add(trialvhitp);
		  std::cerr << "trialvhitp = " << trialvhitp << ",  layerno = "
			    << trialvhitp->GetLayerNumber() << std::endl;
                  truevhittmpp->Add(new U4CDCVectorHit(*trialvhitp, kTRUE));
                  TIter nextht(trialvhitp);
                  U4CDCTrackHit *ht;
                  Int_t ngoodhits = 0;
                  while ((ht = (U4CDCTrackHit *)nextht())) { //poison used hits
                     if (TMath::Power(ht->GetResidual()/xKCNsigmas,2) <
                         trialvhitp->SGX2at(((U4CDCHit *)ht
                                                    ->GetHitPtr())->Ywire())) {
                         ((U4CDCHit *)ht->GetHitPtr())->Lock();
                         ngoodhits++;
                     }
                  }
                  if (ngoodhits >= 4) goto NEXT;
               }
            }
            NEXT: continue;
         }
      }
   }
}

void U4CDCTrackMakerBuf::DebugPrint(const Char_t *opt)
{
        std::cerr << fOutputBuf.GetEntries()
                  << " tracks in this event:" << std::endl;
        TIter next(&fOutputBuf);
        U4CDCTrack *tkp;
        Int_t n = 0;
        while ((tkp = (U4CDCTrack *)next())) {
                std::cerr << std::setw(3) << ++n << " ";
                tkp->DebugPrint(opt);
        }
}
