//*************************************************************************
//* ---------------------
//* U4ITKalHit Class
//* ---------------------
//* (Description)
//* (Requires)
//*     TObjArray
//* (Provides)
//* (Update Record)
//*     2004/08/26  A.Yamaguchi    Original Ver.
//*************************************************************************

#include <iostream>
#include <iomanip>

#include "U4ITKalHit.h"
#if ROOT_VERSION_CODE >= ROOT_VERSION(5,15,0)
#include "TMath.h"
#endif

ClassImp(U4ITKalHit)

//=====================================================================
//* constructor -------------------------------------------------------

U4ITKalHit::U4ITKalHit(Int_t m)
          : U4VKalHit(m) 
{
}
      
U4ITKalHit::U4ITKalHit(const TObject       &hit,
                       const U4ITMeasLayer &ms,
                             Double_t      *x,
                             Double_t      *dx, 
                             Double_t       b,
                             Int_t          m)
          : U4VKalHit(hit, ms,x,dx,b,m)
{
}

U4ITKalHit::U4ITKalHit(const U4ITKalHit &hit)
          : U4VKalHit(hit)
{
}

//=====================================================================
//* destructor --------------------------------------------------------

U4ITKalHit::~U4ITKalHit()
{
}

//=====================================================================
//* XvToMv ------------------------------------------------------------

TKalMatrix U4ITKalHit::XvToMv(const TVector3 &xv, Double_t t0) const
{
   const U4ITMeasLayer &ms
                 = dynamic_cast<const U4ITMeasLayer &>(GetMeasLayer());
   TKalMatrix h  = ms.XvToMv(xv);
   Double_t r    = ms.GetR();
   Double_t phih = h(0,0) / r;
   Double_t phim = (*this)(0,0) / r;
   Double_t dphi = phih - phim;

   static Double_t kPi    = TMath::Pi();
   static Double_t kTwoPi = 2 * kPi;

   while (dphi < -kPi) dphi += kTwoPi;
   while (dphi >  kPi) dphi -= kTwoPi;

   h(0,0)  = r * (phim + dphi);
   h(1,0) += 0.;

   return h;
}

//=====================================================================
//* DebugPrint --------------------------------------------------------

void U4ITKalHit::DebugPrint(Option_t *) const
{
   using namespace std;
   cerr << "------------------- Site Info -------------------------" << endl;

   for (Int_t i=0; i<GetDimension(); i++) {
      Double_t x  = (*this)(i,0);
      Double_t dx = (*this)(i,1);
      cerr << " x[" << i << "] = " << setw(8) << setprecision(5) << x
           << "    "
           << "dx[" << i << "] = " << setw(6) << setprecision(2) << dx
           << setprecision(7)
           << resetiosflags(ios::showpoint)
           << endl;
   }
   cerr << " r    = " << setw(8)
        << static_cast<const U4ITMeasLayer &>(GetMeasLayer()).GetR() << endl;
   cerr << "-------------------------------------------------------" << endl;
}
